' This simple example implements a keyboard controlled ball.
' Arrow keys apply forces To the ball.
' [Ctrl] key reverses gravity force direction.

' After compilation, this source code is turned into a file named 3impactd.dll And
' saved To the 3impactWork folder on your hard disk.
' When 3impactd.exe file (the engine) is launched, it loads this .dll file And
' executes the five functions below, also known as 'callbacks'.
' They are: PreInit(), Init(), Run(), Exit(), PostExit().

' You can fully control the engine by calling creation functions, setting functions
' And transformation functions inside the callbacks. It is that easy! See below.

Strict
Framework _3Impact.Lib

' In the Init() callback you'll see that we create virtual world objects by calling
' creation functions.
' Newly created objects should always be stored in proper 'containers', named
' 'object variables', to allow for subsequent handling.
' Object variables are typically declared at the beginning of the code. Typically, one
' single line of code declares one single variable. A declaration is made of two parts.
' The first part declares the Type of the variable (For example 'CAMERA*') and the
' second part declares its custom (user defined) name (For example 'Camera').
' For simplicity, an Object variable is usually called 'object' (e.g., 'camera object').
' The following are the Object variables we need For this example.
Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()

	   ' This callback is executed once, Right after opening the rendering window Or the
	   ' rendering fullscreen display.
	   ' This callback is typically used To create objects, To set their initial status,
	   ' physics properties, rendering mode, etc.
	
	   ' We always need To create a camera (point of view). Without a camera, nothing can
	   ' be rendered To the screen. The following line calls the iCameraCreate() Function
	   ' And stores the camera Object which it creates in the Object variable named Camera.
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   ' Now that the newly created camera is in the 'Camera' variable, we can handle it.
	   ' For example, we can set its location, inside the virtual space. The Next line of code
	   ' calls the iCameraLocationSet() Function, passing the Camera as first parameter And
	   ' a constant 3d vector as second parameter (the location).
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   ' We also want To specify sun light direction And color.
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   ' Creating objects can be a lengthy operation. We want To display something While
	   ' loading is in progress. So we create a 2d Object (also known as 'sprite') to
	   ' display a 'Loading...' message.
	   ' Note that Object creation will typically require a resource file, as source.
	   ' For example, For sprites, we need an image file And a .x file.
	   ' Custom resources can be created. Refer To the quick start tutorial For details.
	   ' The Next line of code will create a sprite from the loading.x resource, And will
	   ' store it in the variable named 'LoadingMessage'.
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   ' We want the message displayed now, before we start creating (loading) all other objects.
	   iSpriteRender(LoadingMessage,False)
	
	   ' We now create a 3d background For the virtual space.
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   ' The first 3d 'solid' object we create is a piece of terrain. Solid objects
	   ' are called 'bodies'. A body is considered 'solid' because it is collision-response
	   ' capable. In this Case, the body is a polygonal Object (polyhedron).
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   ' Bodies are invisible. They are processed by the engine as invisible-solid objects.
	   ' It means that they cannot be rendered (displayed) by the engine. However, we can
	   ' attach a visible mesh To them, To show where they are And what their shape is.
	   ' Mesh objects are non-solid 3d models. Unlike bodies, they are rendered by the engine.
	   ' With the Next line of code, we create a 3d visible Object from a .x
	   ' resource file. As parameters, we pass the file name For the resource To load And
	   ' the body Object we want To attach the mesh To (Terrain).
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   ' The second body we create is a ball. Unlike the terrain, which is made of polygons,
	   ' this Object is sphere-group based. It is as simple as a ball, so it is a one-ball
	   ' group. See tutorials For details on polyhedron And sphere-group based bodies.
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   ' As a body, the newly created ball is invisible. Let's attach a ball-shaped mesh to it.
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   ' We want the ball at about 5 meters above the ground, at startup.
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   ' By Default, there is no air friction For bodies. We can simulate it by setting
	   ' some damping For the ball.
	   ' See Reference document For details on iBodyDampingSet() instruction And damping.
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   ' We want our ball To cast a shadow onto other objects in the scenery.
	   ' The Next line of code creates a shadow volume mesh from a source .x file
	   ' And attaches it To the Ball body. See Reference document For details on
	   ' iMeshBodyShadowCasterCreate() instruction And shadows.
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	
	   ' By Default, no collision checking is performed For the bodies we have created.
	   ' We have To tell the engine what body couples should be checked For collision.
	   ' The Next line of code creates a so called body-body object.
	   ' Basically, the body-body Object is a data block specifying a body couple And
	   ' a few parameters required by the engine To compute collision-response.
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   ' We have stored the body-body Object For the ball-terrain couple, so that we can
	   ' set collision-response parameters For it. See Reference document For details on them.
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   ' Newly created bodies are disabled (no physics, no collision-response computation).
	   ' The Next two lines of code enable Ball And Terrain bodies.
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)

	   ' We don't want to show the loading message any longer, now that object creation is complete.
	   iSpriteHide(LoadingMessage)
	   ' Newly created meshes are hidden. Let's show all of them.
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	EndMethod
	Method _Run()
	   ' This callback is executed 75 times per second. The first execution happens
	   ' after the Init() callback above is completed.
	
	   ' It means that the engine will repeatedly scan the lines of code from the '{'
	   ' Right below the 'void Run()' line to the '}' right above the 'void Exit()' line.
	
	   ' Each scan from '{' to '}' is called 'dll loop'. It is basically the heartbeat
	   ' of the simulation.
	
	   ' The rendering phase, in which the virtual world is displayed in its current status,
	   ' as seen from the current camera location, happens after each dll loop is complete.
	
	   ' The dll loops Continue Until the [Esc] key is pressed Or the iExit() Function
	   ' is called. Terminating the dll loops basically stops the simulation.
	
	   ' So, we have a piece of terrain And a ball. We want the camera To constantly
	   ' take the ball, wherever it goes. First of all, let's define a vector variable
	   ' To save the current ball position, that is the target To point the camera to.
   	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()

	   ' We put the current ball location into the vector by calling iBodyLocationCM().
	   iBodyLocationCM(Ball,cameratarget)
	   ' Finally, we call a special Function that will point the camera To the target.
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   ' We finally want the ball To be controlled by key presses. The idea is applying
	   ' forces To it according To what arrow keys are pressed by the user.
	   ' First of all, let's define proper variables to store some temporary information
	   ' we need To achieve our goal.
	   Local acceleration:D3DXVECTOR3=_D3DXVECTOR3()
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   ' Linear accelerations (*) are defined by 3d vectors. We have provided a variable
	   ' named 'acceleration' to store the acceleration generated by key presses.
	   ' This vector variable will work as an accumulator. The first Step is resetting it.
	   acceleration.x=0
	   acceleration.y=0
	   acceleration.z=0

	   ' Depending on what keys are pressed, we will add a constant vector To the
	   ' acceleration variable. The length of the vector defines the intensity of
	   ' the acceleration. The direction of the vector defines the direction of it.
	   ' For example, If up-arrow is pressed, we add a 10 meter Long vector
	   ' To the accumulator. Its direction is towards Z-positive world axis.
	   ' It basically means that the ball will be pushed with an acceleration of 10
	   ' meters per second^2 in that direction, when up-arrow is pressed (**).
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   ' (**) The code above basically tends To push the ball along world's X and Z
	   ' axes only, no matter where the camera is taking the scene from. It makes a bit
	   ' difficult For the user To effectively direct the ball.
	   ' We want To re-orientate the resulting acceleration vectors so that they
	   ' are camera relative. For example, we want an acceleration towards X+ axis To be
	   ' re-orientated so that it becomes an acceleration towards the Right of the screen.
	   ' To achieve this, we first store the camera orientation in a quaternion variable.
	   ' Note: most physics engine quantities (locations, forces, torques, etc) can be stored
	   ' in vector variables. To store orientations And rotations we use special variables
	   ' called 'quaternions'.
	   iCameraOrientation(Camera,cameraorientation)
	   ' The Next Function takes a vector (acceleration) And rotates it by using a
	   ' quaternion as a reference. Basically, 'cameraorientation' quaternion is
	   ' the rotation required To change the camera from its natural status (that
	   ' is, looking at Z+ horizon) To its current orientation. We can
	   ' say that it is the current camera orientation, but it is a rotation actually. 
	   ' So, we basically apply that orientation To the given vector. The result
	   ' is that the vector is transformed from world coordinates (absolute) To
	   ' camera coordinates (relative). And it is exactly what we need here.
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   ' Controls are easier now that applied acceleration has been made camera-relative.
	   ' Still we aren't done yet. Controlling the ball when the camera is pitched (for
	   ' example facing downward) isn't very easy. For some reason, it would be easier
	   ' If acceleration vector direction was always parallel To the XZ (ground) plane.
	   ' To constrain the vector To the XZ plane we just have To force its Y
	   ' coordinate To zero. Note that this may reduce vector length (intensity) on
	   ' extreme camera pitching, but it is ok For this simple example.
	   acceleration.y = 0.0
	
	   ' Now that the acceleration vector is set (Or still reset To Null If no
	   ' key was pressed), we can finally apply it To the Ball body.
	   ' (*) We are applying an acceleration And Not a force. Applying a force
	   ' means transferring To a body a defined quantity of energy. So, the resulting
	   ' speed depends on the body mass: a force applied To an heavy body will generate
	   ' a Lower speed respect To the same force applied To a lighter body.
	   ' Applying an acceleration, instead, means transferring To a body the energy required
	   ' To obtain, over time, a defined velocity Or spin, regardless of the body mass.
	   iBodyAccelerationApply(Ball,acceleration)
	
	   ' Finally, we apply a constant acceleration To simulate gravity.
	   ' We also implement an additional control, with the Next lines of code.
	   ' We want To reverse the gravity force (but we should actually name it
	   ' 'gravity acceleration'...) while the [Ctrl] key is pressed.
	   If Not iKeyDown(DIK_LCONTROL)
	      iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else 
	      iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf

	EndMethod
EndType
