' Mesh warping demo. Let's warp the default ball mesh.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib
Import BRL.Basic

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY
Global Envmap:ENVMAP

' define variables To store mesh vertex count And vertex arrays
Global MeshVBSize:Int
Global MeshVBOriginal:MESHVERTEXBUFFER
Global MeshVB:MESHVERTEXBUFFER

' define variables To store shadow caster mesh vertex count And vertex arrays
Global MeshShadowVBSize:Int
Global MeshShadowVertexOriginal:MESHVERTEXBUFFER
Global MeshShadowVB:MESHVERTEXBUFFER

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(23.0,10.0,-150.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(23.0,12.0,-140.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow_lowpoly.x",Ball,20.0)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   Envmap=iEnvMapCreate("default_res\\envmap.dds")
	   iMeshEnvMapReceiverAdd(BallMesh,Envmap)
	   iMeshEnvMapBlendSet(BallMesh,0.20)
	
	   ' Before we can access And change vertex data, we must convert the
	   ' mesh vertex buffer To dynamic.
	   iMeshVertexBufferDynamicMake(BallMesh,True)
	   ' Store the pointer To the list of original vertices in a MESHVERTEX* variable
	   ' named MeshVBOriginal. See Run() callback below For more on this.
	   iMeshVertexBufferOriginal(BallMesh,MeshVBOriginal)
	   ' Store the number of vertices forming the mesh in a variable named MesVBSize.
	   MeshVBSize = iMeshVertexBufferSize(BallMesh)
	
	   ' We are going To warp the shadow caster mesh too, so do similar
	   ' initialization For it. See Run() callback below For more.
	   iMeshVertexBufferDynamicMake(BallMeshShadow,True)
	   iMeshVertexBufferOriginal(BallMeshShadow,MeshShadowVertexOriginal)
	   MeshShadowVBSize = iMeshVertexBufferSize(BallMeshShadow)
	
	   ' Because mesh And shadow caster shapes are slightly different And we don't want
	   ' To see any Self-shadowing artifacts, render the mesh after all shadows are
	   ' painted, which means that the mesh surface will Not receive any cast shadow.
	   iMeshRenderModeSet(BallMesh,D3DBLEND_SRCALPHA,D3DBLEND_INVSRCALPHA,True,True)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	EndMethod
	Method _Run()

	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If (Not iKeyDown(DIK_LCONTROL))
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   ' Meshes are defined by two arrays of data. The first array, called
	   ' 'face buffer', is a list of triangles.  The second array, called
	   ' 'vertex buffer', is a list of vertices. 
	
	   ' In the face array, For each triangle, there are 3 integer values. Each value is
	   ' an index To the vertex buffer. The 3 indices indicate what vertices form the triangle.
	
	   ' In this example we are only manipulating the vertex buffer, as all we need, To
	   ' achieve our mesh warping effect, is displacing the vertices from their original
	   ' location. We are Not going To remove/create faces, so we can ignore face buffers.
	
	   ' Define some variables For our computation
	   Local i:Int
	   Local py:Float,pz:Float,offset:Float
	   Local noriginal:D3DXVECTOR3=_D3DXVECTOR3()
	   ' Define  set variables To be used as warp effect parameters
	   Local amplitude:Float = 0.10
	   Local frequency:Float = 5.0
	
	   ' Before we can access a vertex buffer we must open it.

	   iMeshVertexBufferOpen(BallMesh,MeshVB)
	
	   ' The Function call above puts the vertex buffer 'handle' (the pointer
	   ' To the array of vertices) into the variable named MeshVB.
	
	   ' This way, you can conveniently access data For each vertex in the buffer.
	   ' For example, you can read the X coordinate of the 10th vertex with
	
	   ' MyX = MeshVB[9].p.x
	
	   ' Note that the first vertex is MeshVB[0], so the 10th vertex is MeshVB[9]
	
	   ' You can get the whole location vector (Not just one coordinate) with
	
	   ' MyVector = MeshVB[9].p
	
	   ' You can get the vertex normal vector with
	
	   ' MyNormalVector = MeshVB[9].n
	
	   ' You can get vertex U And V texture-coordinates with
	
	   ' MyU = MeshVB[9].u
	   ' MyV = MeshVB[9].v
	
	   ' Of course you can also write vertex data To the buffer with something like 
	
	   ' MeshVB[9].p.x = 1.75f
	
	
	   ' NOTE: what we have said about MeshVB holds For MeshVBOriginal as well.
	   ' However, While MeshVB points To the actual mesh vertex buffer, MeshVBOriginal
	   ' points To a copy of the actual buffer as it was when the mesh was created.
	   ' Having access To the original vertex buffer is useful To most
	   ' mesh warping algorithms, like the one implemented in this demo.
	
	   ' Unlike actual vertex buffers, original vertex buffers doesn't have to be
	   ' converted To dynamic before they can be accessed. Nor they have To be
	   ' opened. They are system memory (Not video memory), so they can be
	   ' accessed directly.
	
	   ' The program section within the { And the } below, is executed MeshVBSize-times.
	   ' At the first loop, the variable named 'i' is 0. At the second loop it is 1 and so on.
	   ' It means that at first loop we process the first vertex in the buffer, at the second
	   ' loop we process the second vertex And so on, Until the last vertex is processed.
	   For i=0 To MeshVBSize-1
	      ' Compute vertex displacement depending on current vertex's X coordinate
	      ' And current system time.
	      offset = amplitude * sinf((iSystemTimer()+MeshVBOriginal.get_pX(i)*frequency))
	      ' Add the offset To original Y And Z vertex coordinates To find resulting location
	      py = MeshVBOriginal.get_pY(i) + offset
	      pz = MeshVBOriginal.get_pZ(i) + offset
	      ' Write the resulting Y And Z To the actual vertex buffer, hence modifying the mesh.
	      MeshVB.set_pY(i,py)
	      MeshVB.set_pZ(i,pz)
	
	      ' Shading, lighting, envmaps all require that each vertex specify a so called
	      ' 'normal vector'. It is basically the 'direction' of the mesh surface at vertex
	      ' location. The normal 'arrow' typically points outwards (it is normal,
	      ' perpendicular, To the surface). Because we have displaced some vertices,
	      ' we have also changed the surface direction. So we have To accordingly update
	      ' the normal vector For the current vertex in order To guarantee proper
	      ' surface rendering effects like envmap, specular spots, etc.
	
	      ' The following code re-orients the normal vector by using basically the same
	      ' algorithm as the one used For the vertex location above. In this Case,
	      ' because the mesh deformation isn't extreme, updating vertex normals
	      ' isn't vital. Try commenting out the code below to see the difference.
	      noriginal = MeshVBOriginal.getP(i).Plus(MeshVBOriginal.getN(i))
	      offset = amplitude * sinf((iSystemTimer()+noriginal.x)*frequency)
	      MeshVB.set_nY(i,noriginal.y + offset - py)
	      MeshVB.set_nZ(i,noriginal.z + offset - pz)
	   Next
	
	   ' While the buffer is open, the GPU cannot render the mesh.
	   ' Always remember To close the buffer when manipulation is complete!
	   iMeshVertexBufferClose(BallMesh)

	   ' NOTE: shadow caster rendering is computationally expensive (slow). Shadow
	   ' caster warping is even more expensive. Keep it in mind when planning...
	
	   ' The following code warps the shadow caster mesh by using the same
	   ' algorithm used For the visible mesh above.
	
	   iMeshVertexBufferOpen(BallMeshShadow,MeshShadowVB)

	   For i=0 To MeshShadowVBSize-1
	      offset = amplitude * sinf((iSystemTimer()+MeshShadowVertexOriginal.get_pX(i))*frequency)
	      MeshShadowVB.set_pY(i,MeshShadowVertexOriginal.get_pY(i) + offset)
	      MeshShadowVB.set_pZ(i,MeshShadowVertexOriginal.get_pZ(i) + offset)
	   Next
	   iMeshVertexBufferClose(BallMeshShadow)

	   ' After displacing shadow caster mesh vertices, you must recompute the normals by
	   ' calling a special Function, Or face horrible cast shadow artifacts.
	   ' Keep also in mind that extreme warping of shadow caster meshes may lead To
	   ' rendering artifacts anyway, due To limitations in cast shadow algorithms.
	   iMeshShadowCasterNormalsRecompute(BallMeshShadow)
	
	   ' REMARKS
	
	   ' * Vertex position And normal direction, as stored in the vertex buffer,
	   '   is always source-model-center relative.
	   '   You can convert from world-relative coordinates To source-model-center
	   '   relative coordinates with the iVectorWorldToMeshTransform() function.
	
	   ' * Accessing face buffers is very similar To accessing vertex buffers. Just keep in
	   '   mind that the pointer To the buffer is of MESHFACE* type. For example, you call
	   '   iMeshFaceBufferOpen(Mesh,MeshFB) And get the MeshFB pointer. You can Then
	   '   access the 10th face with something like MeshFB[9].v0 (first triangle vertex
	   '   index), MeshFB[9].v1 (second triangle vertex index), MeshFB[9].v2 (second triangle
	   '   vertex index).
	
	   '   Assuming that you have both the vertex And face buffer open, you can, For example,
	   '   access the location vector of the 2nd vertex of the 10th face with
	
	   '   MeshVB[MeshFB[9].v1].p
	
	   ' * Vertex/face buffer manipulation can lead To poor frame rates on some old systems.
	
	   ' * Please consider working with vertex And face buffers an advanced topic.
	EndMethod
EndType

