' Network chat demo.
' While a 3d shape spins in the background, connected users can Type a message And
' press [Enter] To send it To the server, which in turn sends it To all connected
' clients, that display it locally.

' NOTE: in this commented source we assume you understand the NetBallGame code example.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global FontWhite:FONT

Global BallMesh:MESH

Const MAX_MESSAGE_LENGTH:Int = 200
Global MessageString:String
Global TextLine1:String
Global TextLine2:String
Global TextLine3:String

Global RunningAsServer:Int
Global RunningAsClient:Int

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   ' We are going To render a spinning mesh in the background To
	   ' show that chatting can happen without halting the simulation
	   BallMesh=iMeshCreate("default_res\\ball.x")
	   iMeshLocationSet(BallMesh,_D3DXVECTOR3(0.0,0.0,3.5))
	
	   ' Initialize flags And message strings
	   RunningAsServer = False
	   RunningAsClient = False
	   MessageString=""
	   TextLine1=""
	   TextLine2=""
	   TextLine3=""
	
	   ' Let the user decide whether To launch the demo in server Or client mode
	   If (iMessageBoxYesNo("Run the demo in server mode?",0))
	      ' Start the network system in server mode.
	      If (iNetServerStart("Network Chat Demo","The Server",25857,$BCBCBCBC,10000))
		    RunningAsServer = True
	      Else
	         ' an error occurred, terminate
	         iMessageBox("Could not start the server!")
	         iExit()
	      EndIf
	   Else
	      ' Start the network system in client mode.
	      If (iNetClientStart("","The Client",$BCBCBCBC,10000,"NetChat",-1,0,Null))
		    RunningAsClient = True
	      Else
	         ' an error occurred Or the user canceled the connection dialog, terminate
	         iMessageBox("Could not start the client!")
	         iExit()
	      EndIf
	   EndIf
	
	   ' Show the background mesh
	   iMeshShow(BallMesh)
	EndMethod
	
	Method _Run()
	   If (RunningAsServer)
	      ' This If() section is executed only on the server machine.
	      ' It is never executed on a client, Or If the network system isn't running.
	
	      Local messagebuffer:String
	      Local messagetype:Int
	      Local messageplayer:Int
		 Local buf:Byte Ptr
	      ' At every DLL loop, poll the message buffer To
	      ' see If a message from a client has arrived
	      iNetData(Varptr(buf),Varptr(messagetype),Varptr(messageplayer))
		 messagebuffer=String.FromCString(buf)
	      If (messagebuffer.length>0)
	         ' Because 'messagelength' variable is non-zero, we know that a message has arrived
	         ' The 'messagebuffer' variable points to the first byte of the message.
	         ' The 'messagetype' variable is the type of message.
	         ' The 'messageplayer' variable is the player's ID of the sender.
	         If (messagetype = $0ffffff)
	            ' Message Type is 0x0fffffff. In this example, it means that the message is a
	            ' zero-terminated text String (chat message), sent by some remote client user.
	
	            ' Store some space (allocate a String) For chat message String manipulation. 
	            Local playermessage:String
	
	            ' Before sending the received chat String To all connected clients,
	            ' we want To insert the name of the sender at the beginning of it.
	            ' Basically, If the incoming message is from Player1 And reads 'Hello!',
	            ' we are building And sending a String that reads 'Player1> Hello!'.
	
	            ' Retrieve the name of 'messageplayer' and copy it to 'playermessage' string
	            iNetPlayerName(messageplayer,playermessage)
	            ' Append > as separator
	            playermessage:+"> "
	            ' Append the chat message.
	            ' Note that lstrcat() works here only because 'messagebuffer' points to
	            ' a zero-terminated string. Normally, messages are bare sequences of bytes
	            ' that are Not zero-terminated strings (lstrcat() would crash).
	            playermessage:+messagebuffer
	            ' Send the composed chat message To all connected clients.
	            iNetDataSend(playermessage,playermessage.length,$0ffffff,$00000000,True)
	
	            ' Scroll up the chat message lines And add the New message.
	            ' We are showing all messages on server computer too.
	            TextLine1=TextLine2
	            TextLine2=TextLine3
	            TextLine3=playermessage
	
	            ' Release message String manipulation array
	            playermessage=Null
	
	            ' Don't forget to remove the message from the buffer once you are done with it
	            iNetDataDiscard()
	         EndIf
	      EndIf
	   EndIf
	
	   If (RunningAsClient)
	      ' This If() section is executed only on client machines.
	      ' It is never executed on the server, Or If the network system isn't running.
	
	      ' The following code appends typed characters To the current 'MessageString' string.
	      Local c:Byte = iTypedChar(True)
	      If (c)
	         ' Note: characters aren't appended if 'MessageString' is full
	         If (MessageString.length < MAX_MESSAGE_LENGTH)
	            ' 'MessageString' isn't full yet, append the character
	            MessageString:+Chr(c)
	         EndIf
	      EndIf
	      ' The following code removes the last character from
	      ' 'MessageString' when [Backspace] is pressed.
	      If (iKeyClick(DIK_BACKSPACE))
	         If (MessageString.length>0)
	            ' 'MessageString' isn't empty, remove last character
	            MessageString=MessageString[0..MessageString.length-2]
	         EndIf
	      EndIf
	      ' The following code sends 'MessageString' as a
	      ' message To the server when [Enter] is pressed.
	      If (iKeyClick(DIK_RETURN))
	         ' Note: the message isn't sent if 'MessageString' is empty
	         If (MessageString.length>0)
	            ' Send the message To the server
	            iNetDataSend(MessageString,MessageString.length,$0ffffff,$00000000,True)
	            ' Clear 'MessageString'
	            MessageString=""
	         EndIf
	      EndIf
	
	      Local messagebuffer:String
	      Local messagetype:Int
	      Local messageplayer:Int
		 Local buf:Byte Ptr
	      ' At every DLL loop, poll the message buffer To
	      ' see If a message from the server has arrived
	      iNetData(Varptr(buf),Varptr(messagetype),Varptr(messageplayer))
		 messagebuffer=String.FromCString(buf)
	      If (messagebuffer.Length>0)
	         ' iNetData() returned non-zero. It means that a message has arrived
	         ' The 'messagebuffer' variable points to the first byte of the message.
	         ' The 'messagetype' variable is the type of message.
	         ' The 'messageplayer' variable is the player's ID of the sender.
	         If (messagetype = $0ffffff)
	            ' Message Type is 0x0fffffff. In this example, it means that the message is a
	            ' zero-terminated text String (sender name + chat message), sent by the server.
	
	            ' Scroll up the chat message lines And add the New message
	            TextLine1=TextLine2
	            TextLine2=TextLine3
	            TextLine3=messagebuffer
	
	            ' Don't forget to remove the message from the buffer once you are done with it
	            iNetDataDiscard()
	         EndIf
	      EndIf
	   EndIf
	
	   ' Render current, locally typed message String
	   ' Note that, in this example, the server user cannot Type any message,
	   ' so 'MessageString is always empty when the program runs on the server
	   iPrintCentered(MessageString,_D3DXVECTOR2(0.0,0.0),_D3DXVECTOR2(1.0,1.0),0.1,FontWhite,0.0)
	
	   ' Render the three lines of received chat messages 
	   iPrintCentered(TextLine1,_D3DXVECTOR2(0.0,-10.0),_D3DXVECTOR2(0.5,0.5),0.05,FontWhite,0.0)
	   iPrintCentered(TextLine2,_D3DXVECTOR2(0.0,-10.75),_D3DXVECTOR2(0.5,0.5),0.05,FontWhite,0.0)
	   iPrintCentered(TextLine3,_D3DXVECTOR2(0.0,-11.5),_D3DXVECTOR2(0.5,0.5),0.05,FontWhite,0.0)
	
	   ' Spin the background mesh
	   iMeshOrientationAdd(BallMesh,_D3DXVECTOR3(0.0,1.0,0.0),30.0)
	EndMethod
	
	Method _Exit()
	   ' The network system must always be stopped before exiting the application
	   iNetStop()
	EndMethod
EndType
