' Simple particle demo

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global TerrainMesh:MESH
Global FontWhite:FONT

Global Particle:PARTICLE
Global Emitter:EMITTER

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,1.0,-7.0))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   TerrainMesh=iMeshCreate("default_res\\terrain.x")
	
	   ' A particle is basically an image that is rendered at a defined
	   ' 3d location, so that it always faces the camera.
	
	   ' The image is typically a power-of-two sized picture (e.g. 256x256,
	   ' 256x16, 8x64, etc) with alpha information, saved as a .dds file
	   ' Or another alpha-capable image file format like .tga.
	
	   ' The alpha information allows For each pixel in the image To have
	   ' a certain degree of transparency, anywhere from totally transparent
	   ' To totally opaque.
	
	   ' Many paint tools support .dds file format. A free one is provided by
	   ' Microsoft And is included To DirectX SDK installation. Its name is
	   ' DirectX Texture Tool And can also be found over the Internet as a
	   ' smaller installation file.
	
	   ' Particles are emitted into the scenery at a specified location And
	   ' persist there some time. During their lifetime, they can move
	   ' at a certain speed, be affected by air friction And gravity,
	   ' And can smoothly change size And color.
	
	   ' Rendering many moving particles about a defined 3d location gives the
	   ' impression of a 3d mass of some animated substance.
	   ' Properly setting particle properties allows you To simulate effects
	   ' like smoke, fire, water, energy And much more.
	 
	   ' Creates the dust particle from an image file, also setting To 3000
	   ' the maximum number of particles that can be simultaneously rendered.
	   ' Try also the following pre-made .dds files: particle_dust.dds,
	   ' particle_clot.dds, particle_spots.dds, particle_sharp.dds, particle_crumbs.dds
	   Particle=iParticleCreate("default_res\\particle_globe.dds",3000)
	   ' Sets the rendering mode To normal. Try setting it To 1 For an interesting
	   ' burnout effect that is typically used To render energy.
	   iParticleRenderModeSet(Particle,0)
	   ' Sets no gravity effect For the particles. Try simulating Earth gravity by
	   ' setting it To D3DXVECTOR3(0.0,-9.80665,0.0).
	   iParticleGravitySet(Particle,_D3DXVECTOR3(0.0,0.0,0.0))
	   ' Sets no air resistance.
	   iParticleAirResistanceSet(Particle,-1.0)
	   ' Sets color change from yellow To red
	   iParticleColorSet(Particle,_D3DXVECTOR3(1.0,1.0,0.0),_D3DXVECTOR3(1.0,0.0,0.0))
	   ' Sets the life time For each particle To one second.
	   iParticleLifeTimeSet(Particle,1.0)
	
	   ' Creates an emitter For the particle. Note that you can create many emitters
	   ' with different properties For the same particle, If you wish.
	   Emitter=iParticleEmitterCreate(Particle)
	   ' Places the emitter in front of the camera
	   iParticleEmitterLocationSet(Emitter,_D3DXVECTOR3(0.0,2.0,0.0))
	   ' Particles are emitted To random directions at random speed, but within
	   ' defined ranges. The following line sets the ranges For X, Y And Z component
	   ' of the direction/speed vector. In this Case, For each emitted particle,
	   ' a direction vector is automatically computed by sorting its X,Y,Z components
	   ' between -1.0f And 1.0f. Try changing the ranges To constrain resulting vector
	   ' direction between specific compasses And/Or achieve different particle velocities.
	   iParticleEmitterDirectionSet(Emitter,_D3DXVECTOR3(-1.0,-1.0,-1.0),_D3DXVECTOR3(1.0,1.0,1.0))
	   ' Sets initial And Final size For the particles so that they start
	   ' 1.0f meter wide And do Not change over time. Try setting different
	   ' sizes To experiment with particle scale variation.
	   iParticleEmitterScaleSet(Emitter,_D3DXVECTOR2(1.0,1.0),_D3DXVECTOR2(1.0,1.0))
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   ' Don't forget to show your particle object!
	   iParticleShow(Particle)
	EndMethod
	
	Method _Run()
	   iPrintCentered("Hold [Space] pressed to emit",_D3DXVECTOR2(0.0,-11.5),_D3DXVECTOR2(0.5,0.5),0.1,FontWhite,0.0)
	
	   If (iKeyDown(DIK_SPACE))
	      ' Emits the particles from the specified emitter While the [Space]
	      ' key is pressed. One single particle is emitted per second.
	      ' Try increasing the emission frequency (second parameter) And
	      ' changing the initial particle opacity (third parameter) anywhere
	      ' within 0.0f (invisible) And 1.0f.
	      iParticleEmitterEmit(Emitter,1.0,1.0)
	   EndIf
	EndMethod

EndType
