' Simple keyboard controlled car.
' Controls: arrow keys.

' Don't forget to see Reference.txt document for details on functions called in this example! 

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.


Strict
Framework _3Impact.Lib

' declare Object variables
Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global CarBody:BODY
Global CarBodyMesh:MESH
Global CarBodyMeshShadow:MESH
Global CarWheelFL:BODY
Global CarWheelFLMesh:MESH
Global CarWheelFLMeshShadow:MESH
Global CarWheelFR:BODY
Global CarWheelFRMesh:MESH
Global CarWheelFRMeshShadow:MESH
Global CarWheelRL:BODY
Global CarWheelRLMesh:MESH
Global CarWheelRLMeshShadow:MESH
Global CarWheelRR:BODY
Global CarWheelRRMesh:MESH
Global CarWheelRRMeshShadow:MESH
Global CarBodyTerrain:BODYBODY
Global CarWheelFLTerrain:BODYBODY
Global CarWheelFRTerrain:BODYBODY
Global CarWheelRLTerrain:BODYBODY
Global CarWheelRRTerrain:BODYBODY
Global CarWheelFLJoint:WHEEL
Global CarWheelFRJoint:WHEEL
Global CarWheelRLJoint:WHEEL
Global CarWheelRRJoint:WHEEL
' declare parameter variables
Global SideFriction:Float
Global RollingWiseFriction:Float
Global SuspensionSpring:Float
Global SuspensionDamping:Float
Global SteerAngle:Float
Global Spin:Float
Global WheelSpin:Float
Global WheelPower:Float

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   ' create And set camera
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraFovSet(Camera,35.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(23.0,10.0,-100.0))
	   ' set sunlight
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	   ' create And show 'loading...' sprite
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	   ' create static background
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	   ' create terrain static body And mesh
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   ' create dynamic car-body body And mesh
	   CarBody=iBodySGCreate("simplecar_res\\carbody_.spg",0.13)
	   CarBodyMesh=iBodyMeshCreate("simplecar_res\\carbody.x",CarBody)
	   ' set initial car position
	   iBodyLocationSet(CarBody,_D3DXVECTOR3(43.0,5.0,-100.0),True)
	   ' a little bit of damping is always good To prevent excessive speed And spin
	   iBodyDampingSet(CarBody,0.0,0.999,0.0,0.995)
	   ' create car-body shadow mesh
	   CarBodyMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carbody_shadow.x",CarBody,500.0)
	
	   ' enable car-terrain collision checking (create collision couple)
	   CarBodyTerrain=iBodyBodyCreate(CarBody,Terrain)
	   iBodyBodyFrictionSet(CarBodyTerrain,2.5)
	
	   ' ********************************************************
	   ' SIMULATION PARAMETERS: you can adjust car-dynamics here!
	
	   ' the friction produced by wheels, when they move sideways respect To terrain
	   SideFriction = 7.0
	   ' the friction produced by wheels, when they move parallel To rolling direction
	   RollingWiseFriction = 3.0
	   ' suspension strength
	   SuspensionSpring = 10.0
	   ' suspension hardness
	   SuspensionDamping = 1.0
	   ' maximum wheel spin (degrees per second)
	   WheelSpin = 1600.0
	   ' maximum wheel power applied To reach And keep maximum speed
	   WheelPower = 1.0
	
	   ' ********************************************************
	
	   ' initialize steering And throttle
	   SteerAngle = 0.0
	   Spin = 0.0
	
	   ' create FRONT-Left wheel dynamic body, mesh And shadow mesh
	   CarWheelFL=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	   CarWheelFLMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelFL)
	   CarWheelFLMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelFL,500.0)
	   ' enable terrain-wheel collision checking
	   CarWheelFLTerrain=iBodyBodyCreate(CarWheelFL,Terrain)
	   ' enable split-friction mode
	   iBodyBodySplitFrictionSet(CarWheelFLTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	   ' create wheel joint (attach wheel-body To car-body) And set suspension parameters
	   CarWheelFLJoint = iWheelCreate(CarBody,CarWheelFL,_D3DXVECTOR3(-1.12,0.16,1.22),Null)
	   iWheelSuspensionSet(CarWheelFLJoint,SuspensionSpring,SuspensionDamping)
	
	   ' create FRONT-Right wheel dynamic body, mesh And shadow mesh
	   CarWheelFR=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	   CarWheelFRMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelFR)
	   CarWheelFRMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelFR,500.0)
	   ' enable terrain-wheel collision checking
	   CarWheelFRTerrain=iBodyBodyCreate(CarWheelFR,Terrain)
	   ' enable split-friction mode
	   iBodyBodySplitFrictionSet(CarWheelFRTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	   ' create wheel joint (attach wheel-body To car-body) And set suspension parameters
	   CarWheelFRJoint = iWheelCreate(CarBody,CarWheelFR,_D3DXVECTOR3(1.12,0.16,1.22),Null)
	   iWheelSuspensionSet(CarWheelFRJoint,SuspensionSpring,SuspensionDamping)
	
	   ' create REAR-Left wheel dynamic body, mesh And shadow mesh
	   CarWheelRL=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	   CarWheelRLMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelRL)
	   CarWheelRLMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelRL,500.0)
	   ' enable terrain-wheel collision checking
	   CarWheelRLTerrain=iBodyBodyCreate(CarWheelRL,Terrain)
	   ' enable split-friction mode
	   iBodyBodySplitFrictionSet(CarWheelRLTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	   ' create wheel joint (attach wheel-body To car-body) And set suspension parameters
	   CarWheelRLJoint = iWheelCreate(CarBody,CarWheelRL,_D3DXVECTOR3(-1.12,0.16,-1.10),Null)
	   iWheelSuspensionSet(CarWheelRLJoint,SuspensionSpring,SuspensionDamping)
	
	   ' create REAR-Right wheel dynamic body, mesh And shadow mesh
	   CarWheelRR=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	   CarWheelRRMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelRR)
	   CarWheelRRMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelRR,500.0)
	   ' enable terrain-wheel collision checking
	   CarWheelRRTerrain=iBodyBodyCreate(CarWheelRR,Terrain)
	   ' enable split-friction mode
	   iBodyBodySplitFrictionSet(CarWheelRRTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	   ' create wheel joint (attach wheel-body To car-body) And set suspension parameters
	   CarWheelRRJoint = iWheelCreate(CarBody,CarWheelRR,_D3DXVECTOR3(1.12,0.16,-1.10),Null)
	   iWheelSuspensionSet(CarWheelRRJoint,SuspensionSpring,SuspensionDamping)
	
	   ' enable all bodies
	   iBodyEnable(Terrain)
	   iBodyEnable(CarBody)
	   iBodyEnable(CarWheelFL)
	   iBodyEnable(CarWheelFR)
	   iBodyEnable(CarWheelRL)
	   iBodyEnable(CarWheelRR)
	   ' hide 'loading...' sprite
	   iSpriteHide(LoadingMessage)
	   ' show static background And all meshes
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(CarBodyMesh)
	   iMeshShow(CarBodyMeshShadow)
	   iMeshShow(CarWheelFLMesh)
	   iMeshShow(CarWheelFLMeshShadow)
	   iMeshShow(CarWheelFRMesh)
	   iMeshShow(CarWheelFRMeshShadow)
	   iMeshShow(CarWheelRLMesh)
	   iMeshShow(CarWheelRLMeshShadow)
	   iMeshShow(CarWheelRRMesh)
	   iMeshShow(CarWheelRRMeshShadow)
	EndMethod
	Method _Run()
	   ' update camera orientation so that it always takes the car
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(CarBody,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   ' If Left/Right arrow keys are pressed, progressively increase/decrease the SteerAngle
	   ' variable (within -20 / 20 degrees).
	   ' If neither Left nor Right keys are pressed, progressively adjust SteerAngle variable
	   ' toward zero, in order To perform steering auto-centering.
	   If (iKeyDown(DIK_LEFT))
	      iFloatTendTo(SteerAngle,20.0,0.3)
	   Else If (iKeyDown(DIK_RIGHT))
	      iFloatTendTo(SteerAngle,-20.0,0.3)
	   Else 
		 iFloatTendTo(SteerAngle,0.0,0.2)
	   EndIf
	
	   ' apply current SteerAngle To front wheels (set wheel joints steering parameter)
	   iWheelSteeringAngleSet(CarWheelFLJoint,SteerAngle)
	   iWheelSteeringAngleSet(CarWheelFRJoint,SteerAngle)
	
	   ' If up/down arrow keys are pressed, progressively decrease/increase the Spin
	   ' variable (within -WheelSpin / WheelSpin).
	   ' If neither up nor down keys are pressed, progressively adjust Spin variable
	   ' toward zero, in order To perform throttle auto-returning
	   If (iKeyDown(DIK_UP))
	      iFloatTendTo(Spin,-WheelSpin,0.4)
	   Else If (iKeyDown(DIK_DOWN))
	      iFloatTendTo(Spin,WheelSpin,0.4)
	   Else
		 iFloatTendTo(Spin,0.0,0.1)
	   EndIf
	
	   ' wheel power is controlled by two factors: the maximum spin permitted And the torque
	   ' intensity used To reach And keep such speed.
	   ' In order To simulate a simple one-gear system, in which wheel spin And power are
	   ' proportional To throttle depression, we let the user directly tune the
	   ' maximum spin with up/down arrow keys (previous section above).
	   ' Then we compute the actual wheel power (torque) so that it is proportional To
	   ' such user-adjusted Spin
	   Local power:Float = iFloatInterpolate(iFloatAbs(Spin),0.0,WheelSpin,WheelPower*0.1,WheelPower,False)
	
	   ' However, If a [Ctrl] key is pressed, force Spin And power variables back To zero And 1.5f
	   ' respectively, To simulate brakes. The higher the power value, the stronger the brakes
	   If ((iKeyDown(DIK_LCONTROL)) Or (iKeyDown(DIK_RCONTROL)))
	      Spin = 0.0
	      power = 1.5
	   EndIf
	
	   ' apply user-adjusted spin And computed power To all 4 wheels
	   iWheelSpinSet(CarWheelFLJoint,Spin)
	   iWheelSpinSet(CarWheelFRJoint,Spin)
	   iWheelSpinSet(CarWheelRLJoint,Spin)
	   iWheelSpinSet(CarWheelRRJoint,Spin)
	   iWheelPowerSet(CarWheelFLJoint,power)
	   iWheelPowerSet(CarWheelFRJoint,power)
	   iWheelPowerSet(CarWheelRLJoint,power)
	   iWheelPowerSet(CarWheelRRJoint,power)
	
	   ' apply gravity acceleration To car-body And wheel-bodies
	   iBodyAccelerationApply(CarBody,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   iBodyAccelerationApply(CarWheelFL,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   iBodyAccelerationApply(CarWheelFR,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   iBodyAccelerationApply(CarWheelRL,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   iBodyAccelerationApply(CarWheelRR,_D3DXVECTOR3(0.0,-9.80665,0.0))
	
	   iWindowTitleSet(iFrameRate(True))	
	EndMethod
EndType





