'  Car following a predefined track, consisting of five checkpoints
'  Controls: "A" To switch between auto controlled car, And manual drive
'            Arrow keys To control car when driving manually
'            Left Ctrl To break when driving manually

'  Source code by Stefan Nilsson
'  04/03/2005

'  This source code can be freely distributed by any means.

'  THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

' declare Object variables
Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global CarBody:BODY
Global CarBodyMesh:MESH
Global CarBodyMeshShadow:MESH
Global CarWheelFL:BODY
Global CarWheelFLMesh:MESH
Global CarWheelFLMeshShadow:MESH
Global CarWheelFR:BODY
Global CarWheelFRMesh:MESH
Global CarWheelFRMeshShadow:MESH
Global CarWheelRL:BODY
Global CarWheelRLMesh:MESH
Global CarWheelRLMeshShadow:MESH
Global CarWheelRR:BODY
Global CarWheelRRMesh:MESH
Global CarWheelRRMeshShadow:MESH
Global CarBodyTerrain:BODYBODY
Global CarWheelFLTerrain:BODYBODY
Global CarWheelFRTerrain:BODYBODY
Global CarWheelRLTerrain:BODYBODY
Global CarWheelRRTerrain:BODYBODY
Global CarWheelFLJoint:WHEEL
Global CarWheelFRJoint:WHEEL
Global CarWheelRLJoint:WHEEL
Global CarWheelRRJoint:WHEEL

'  Declare a few checkpoints. We use 5 of them in this example
Global CP:BODY[5]
Global CPMesh:MESH[5]
Global CarBodyCP:BODYBODY[5]

' declare parameter variables
Global SideFriction:Float
Global RollingWiseFriction:Float
Global SuspensionSpring:Float
Global SuspensionDamping:Float
Global SteerAngle:Float
Global Spin:Float
Global WheelSpin:Float
Global WheelPower:Float

'  Different modes
Const CONTROL_CAR:Int =  0   '  User controls the car as usual
Const AUTO_CAR:Int =  1   '  Car is auto controlled
Global mode:Int = AUTO_CAR     '  Default To auto controlled

'  Checkpoint To run For
Global checkpoint:Int = 0

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   ' create And set camera
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraFovSet(Camera,35.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(23.0,10.0,-100.0))
	   ' set sunlight
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	   ' create And show 'loading...' sprite
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	   ' create static background
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	   ' create terrain static body And mesh
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   ' create dynamic car-body body And mesh
	   CarBody=iBodySGCreate("simplecar_res\\carbody_.spg",0.13)
	   CarBodyMesh=iBodyMeshCreate("simplecar_res\\carbody.x",CarBody)
	   ' set initial car position
	   iBodyLocationSet(CarBody,_D3DXVECTOR3(43.0,5.0,-100.0),True)
	   ' a little bit of damping is always good To prevent excessive speed And spin
	   iBodyDampingSet(CarBody,0.0,0.999,0.0,0.995)
	   ' create car-body shadow mesh
	   CarBodyMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carbody_shadow.x",CarBody,500.0)
	
	   ' enable car-terrain collision checking (create collision couple)
	   CarBodyTerrain=iBodyBodyCreate(CarBody,Terrain)
	   iBodyBodyFrictionSet(CarBodyTerrain,2.5)
	
	   '  Create a few checkpoints
	   CP[0]=iBodySGCreate("default_res\\ball_.spg",0.13)
	   CPMesh[0]=iBodyMeshCreate("default_res\\ball.x",CP[0])
	   iMeshAlphaSet(CPMesh[0],0.6)
	   iBodyLocationSet(CP[0],_D3DXVECTOR3(80.0,2.0,-107.0),True)
	
	   CP[1]=iBodySGCreate("default_res\\ball_.spg",0.13)
	   CPMesh[1]=iBodyMeshCreate("default_res\\ball.x",CP[1])
	   iMeshAlphaSet(CPMesh[1],0.2)
	   iBodyLocationSet(CP[1],_D3DXVECTOR3(112.0,2.0,-84.0),True)
	
	   CP[2]=iBodySGCreate("default_res\\ball_.spg",0.13)
	   CPMesh[2]=iBodyMeshCreate("default_res\\ball.x",CP[2])
	   iMeshAlphaSet(CPMesh[2],0.2)
	   iBodyLocationSet(CP[2],_D3DXVECTOR3(23.0,2.0,-2.0),True)
	
	   CP[3]=iBodySGCreate("default_res\\ball_.spg",0.13)
	   CPMesh[3]=iBodyMeshCreate("default_res\\ball.x",CP[3])
	   iMeshAlphaSet(CPMesh[3],0.2)
	   iBodyLocationSet(CP[3],_D3DXVECTOR3(-15.0,1.0,-2.0),True)
	
	   CP[4]=iBodySGCreate("default_res\\ball_.spg",0.13)
	   CPMesh[4]=iBodyMeshCreate("default_res\\ball.x",CP[4])
	   iMeshAlphaSet(CPMesh[4],0.2)
	   iBodyLocationSet(CP[4],_D3DXVECTOR3(26.0,4.0,-77.0),True)
	
	   '  Enable car - checkpoint collision...
	   CarBodyCP[0]=iBodyBodyCreate(CarBody,CP[0])
	   CarBodyCP[1]=iBodyBodyCreate(CarBody,CP[1])
	   CarBodyCP[2]=iBodyBodyCreate(CarBody,CP[2])
	   CarBodyCP[3]=iBodyBodyCreate(CarBody,CP[3])
	   CarBodyCP[4]=iBodyBodyCreate(CarBody,CP[4])
	
	   '  ...but disable interaction...
	   iBodyBodyCollisionResponseEnable(CarBodyCP[0], False)
	   iBodyBodyCollisionResponseEnable(CarBodyCP[1], False)
	   iBodyBodyCollisionResponseEnable(CarBodyCP[2], False)
	   iBodyBodyCollisionResponseEnable(CarBodyCP[3], False)
	   iBodyBodyCollisionResponseEnable(CarBodyCP[4], False)
	   
	   '  ...but enable feedback though
	   iBodyBodyFeedbackEnable(CarBodyCP[0], -1, -1, True)
	   iBodyBodyFeedbackEnable(CarBodyCP[1], -1, -1, True)
	   iBodyBodyFeedbackEnable(CarBodyCP[2], -1, -1, True)
	   iBodyBodyFeedbackEnable(CarBodyCP[3], -1, -1, True)
	   iBodyBodyFeedbackEnable(CarBodyCP[4], -1, -1, True)
	
	   ' ********************************************************
	   ' SIMULATION PARAMETERS: you can adjust car-dynamics here!
	
	   ' the friction produced by wheels, when they move sideways respect To terrain
	   SideFriction = 7.0
	   ' the friction produced by wheels, when they move parallel To rolling direction
	   RollingWiseFriction = 3.0
	   ' suspension strength
	   SuspensionSpring = 10.0
	   ' suspension hardness
	   SuspensionDamping = 1.0
	   ' maximum wheel spin (degrees per second)
	   WheelSpin = 1600.0
	   ' maximum wheel power applied To reach And keep maximum speed
	   WheelPower = 1.0
	
	   ' ********************************************************
	
	   ' initialize steering And throttle
	   SteerAngle = 0.0
	   Spin = 0.0
		
	   ' create FRONT-Left wheel dynamic body, mesh And shadow mesh
	   CarWheelFL=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	   CarWheelFLMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelFL)
	   CarWheelFLMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelFL,500.0)
	   ' enable terrain-wheel collision checking
	   CarWheelFLTerrain=iBodyBodyCreate(CarWheelFL,Terrain)
	   ' enable split-friction mode
	   iBodyBodySplitFrictionSet(CarWheelFLTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	   ' create wheel joint (attach wheel-body To car-body) And set suspension parameters
	   CarWheelFLJoint = iWheelCreate(CarBody,CarWheelFL,_D3DXVECTOR3(-1.12,0.16,1.22),Null)
	   iWheelSuspensionSet(CarWheelFLJoint,SuspensionSpring,SuspensionDamping)
	
	   ' create FRONT-Right wheel dynamic body, mesh And shadow mesh
	   CarWheelFR=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	   CarWheelFRMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelFR)
	   CarWheelFRMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelFR,500.0)
	   ' enable terrain-wheel collision checking
	   CarWheelFRTerrain=iBodyBodyCreate(CarWheelFR,Terrain)
	   ' enable split-friction mode
	   iBodyBodySplitFrictionSet(CarWheelFRTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	   ' create wheel joint (attach wheel-body To car-body) And set suspension parameters
	   CarWheelFRJoint = iWheelCreate(CarBody,CarWheelFR,_D3DXVECTOR3(1.12,0.16,1.22),Null)
	   iWheelSuspensionSet(CarWheelFRJoint,SuspensionSpring,SuspensionDamping)
	
	   ' create REAR-Left wheel dynamic body, mesh And shadow mesh
	   CarWheelRL=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	   CarWheelRLMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelRL)
	   CarWheelRLMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelRL,500.0)
	   ' enable terrain-wheel collision checking
	   CarWheelRLTerrain=iBodyBodyCreate(CarWheelRL,Terrain)
	   ' enable split-friction mode
	   iBodyBodySplitFrictionSet(CarWheelRLTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	   ' create wheel joint (attach wheel-body To car-body) And set suspension parameters
	   CarWheelRLJoint = iWheelCreate(CarBody,CarWheelRL,_D3DXVECTOR3(-1.12,0.16,-1.10),Null)
	   iWheelSuspensionSet(CarWheelRLJoint,SuspensionSpring,SuspensionDamping)
	
	   ' create REAR-Right wheel dynamic body, mesh And shadow mesh
	   CarWheelRR=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	   CarWheelRRMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelRR)
	   CarWheelRRMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelRR,500.0)
	   ' enable terrain-wheel collision checking
	   CarWheelRRTerrain=iBodyBodyCreate(CarWheelRR,Terrain)
	   ' enable split-friction mode
	   iBodyBodySplitFrictionSet(CarWheelRRTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	   ' create wheel joint (attach wheel-body To car-body) And set suspension parameters
	   CarWheelRRJoint = iWheelCreate(CarBody,CarWheelRR,_D3DXVECTOR3(1.12,0.16,-1.10),Null)
	   iWheelSuspensionSet(CarWheelRRJoint,SuspensionSpring,SuspensionDamping)
	
	   ' enable all bodies
	   iBodyEnable(Terrain)
	   iBodyEnable(CarBody)
	   iBodyEnable(CarWheelFL)
	   iBodyEnable(CarWheelFR)
	   iBodyEnable(CarWheelRL)
	   iBodyEnable(CarWheelRR)
	   iBodyEnable(CP[0])
	   iBodyEnable(CP[1])
	   iBodyEnable(CP[2])
	   iBodyEnable(CP[3])
	   iBodyEnable(CP[4])
	   ' hide 'loading...' sprite
	   iSpriteHide(LoadingMessage)
	   ' show static background And all meshes
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(CarBodyMesh)
	   iMeshShow(CarBodyMeshShadow)
	   iMeshShow(CarWheelFLMesh)
	   iMeshShow(CarWheelFLMeshShadow)
	   iMeshShow(CarWheelFRMesh)
	   iMeshShow(CarWheelFRMeshShadow)
	   iMeshShow(CarWheelRLMesh)
	   iMeshShow(CarWheelRLMeshShadow)
	   iMeshShow(CarWheelRRMesh)
	   iMeshShow(CarWheelRRMeshShadow)
	   iMeshShow(CPMesh[0])
	   iMeshShow(CPMesh[1])
	   iMeshShow(CPMesh[2])
	   iMeshShow(CPMesh[3])
	   iMeshShow(CPMesh[4])
	EndMethod
	Method _Run()
	   ' update camera orientation so that it always takes the car
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(CarBody,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   '  Use button "A" To swith between auto controlled car, And manual drive
	   If(iKeyClick(DIK_A))
	      If(mode = CONTROL_CAR)
	         mode = AUTO_CAR
	      Else
	         mode = CONTROL_CAR
	      EndIf
	   EndIf
	
	   '  Check To see If our car has reached the checkpoint
	   If(iBodyBodyFeedback(CarBodyCP[checkpoint], -1, -1, Null, Null, Null, Null, Null, Null))
	      '  Checkpoint reached! Switch checkpoint!
	      iMeshAlphaSet(CPMesh[checkpoint],0.2)  '  Diffuse old checkpoint
	      checkpoint = (checkpoint + 1) Mod 5
	      iMeshAlphaSet(CPMesh[checkpoint],0.6)  '  Intensify current checkpoint
	   EndIf
	
	   If(mode = CONTROL_CAR)
	      ' If Left/Right arrow keys are pressed, progressively increase/decrease the SteerAngle
	      ' variable (within -20 / 20 degrees).
	      ' If neither Left nor Right keys are pressed, progressively adjust SteerAngle variable
	      ' toward zero, in order To perform steering auto-centering.
	      If (iKeyDown(DIK_LEFT))
	         iFloatTendTo(SteerAngle,20.0,0.3)
	      ElseIf (iKeyDown(DIK_RIGHT))
	         iFloatTendTo(SteerAngle,-20.0,0.3)
	      Else
			 iFloatTendTo(SteerAngle,0.0,0.2)
		 EndIf
	      ' apply current SteerAngle To front wheels (set wheel joints steering parameter)
	      iWheelSteeringAngleSet(CarWheelFLJoint,SteerAngle)
	      iWheelSteeringAngleSet(CarWheelFRJoint,SteerAngle)
	
	      ' If up/down arrow keys are pressed, progressively decrease/increase the Spin
	      ' variable (within -WheelSpin / WheelSpin).
	      ' If neither up nor down keys are pressed, progressively adjust Spin variable
	      ' toward zero, in order To perform throttle auto-returning
	      If (iKeyDown(DIK_UP))
	         iFloatTendTo(Spin,-WheelSpin,0.4)
	      ElseIf (iKeyDown(DIK_DOWN))
	         iFloatTendTo(Spin,WheelSpin,0.4)
	      Else 
			iFloatTendTo(Spin,0.0,0.1)
		 EndIf
	
	      ' wheel power is controlled by two factors: the maximum spin permitted And the torque
	      ' intensity used To reach And keep such speed.
	      ' In order To simulate a simple one-gear system, in which wheel spin And power are
	      ' proportional To throttle depression, we let the user directly tune the
	      ' maximum spin with up/down arrow keys (previous section above).
	      ' Then we compute the actual wheel power (torque) so that it is proportional To
	      ' such user-adjusted Spin
	      Local power:Float = iFloatInterpolate(iFloatAbs(Spin),0.0,WheelSpin,WheelPower*0.1,WheelPower,False)
	
	      ' However, If a [Ctrl] key is pressed, force Spin And power variables back To zero And 1.5f
	      ' respectively, To simulate brakes. The higher the power value, the stronger the brakes
	      If ((iKeyDown(DIK_LCONTROL)) Or (iKeyDown(DIK_RCONTROL)))
	         Spin = 0.0
	         power = 1.5
	      EndIf
	
	      ' apply user-adjusted spin And computed power To all 4 wheels
	      iWheelSpinSet(CarWheelFLJoint,Spin)
	      iWheelSpinSet(CarWheelFRJoint,Spin)
	      iWheelSpinSet(CarWheelRLJoint,Spin)
	      iWheelSpinSet(CarWheelRRJoint,Spin)
	      iWheelPowerSet(CarWheelFLJoint,power)
	      iWheelPowerSet(CarWheelFRJoint,power)
	      iWheelPowerSet(CarWheelRLJoint,power)
	      iWheelPowerSet(CarWheelRRJoint,power)
	   Else
	      '  mode == AUTO_CAR
	      
	      '  Get location of current checkpoint
	      Local cp:BODY = CP[checkpoint]
	      Local cpLocation:D3DXVECTOR3=_D3DXVECTOR3()
	      iBodyLocation(cp, cpLocation)
	
	      '  Tranform the coordinates For the checkpoint To coordinates relative To the car
	      Local relativeCpLocation:D3DXVECTOR3=_D3DXVECTOR3()
	      iVectorWorldToBodyTransform(relativeCpLocation, cpLocation, CarBody)
	
	      '  Calculate the angle between cars current orientation ((0, 0, 1) relative To the car)
	      relativeCpLocation.y = 0.0 '  Ignore difference in height
	      Local angle:Float = iVectorsAngle(_D3DXVECTOR3(0.0, 0.0, 1.0), relativeCpLocation)
	      If(relativeCpLocation.x > 0)
	         '  Since iVectorsAngle returns Abs(angle), use the x coordinate For the checkpoint
	         '  To determine If the angle should be negative Or Not
	         angle = angle * -1.0
	      EndIf
	      
	      '  Turn wheels, but make sure that we stay within +-20 degrees
	      iFloatClamp(angle, -20.0, 20.0)
	      iFloatTendTo(SteerAngle, angle, 0.2)
	
	      '  Apply current SteerAngle To front wheels (set wheel joints steering parameter)
	      iWheelSteeringAngleSet(CarWheelFLJoint,SteerAngle)
	      iWheelSteeringAngleSet(CarWheelFRJoint,SteerAngle)
	
	      '  Pedal To the metal!
	      iFloatTendTo(Spin,-WheelSpin,0.4)
	
	      ' wheel power is controlled by two factors: the maximum spin permitted And the torque
	      ' intensity used To reach And keep such speed.
	      ' In order To simulate a simple one-gear system, in which wheel spin And power are
	      ' proportional To throttle depression, we let the user directly tune the
	      ' maximum spin with up/down arrow keys (previous section above).
	      ' Then we compute the actual wheel power (torque) so that it is proportional To
	      ' such user-adjusted Spin
	      Local power:Float = iFloatInterpolate(iFloatAbs(Spin),0.0,WheelSpin,WheelPower*0.1,WheelPower,False)
	
	      ' apply user-adjusted spin And computed power To all 4 wheels
	      iWheelSpinSet(CarWheelFLJoint,Spin)
	      iWheelSpinSet(CarWheelFRJoint,Spin)
	      iWheelSpinSet(CarWheelRLJoint,Spin)
	      iWheelSpinSet(CarWheelRRJoint,Spin)
	      iWheelPowerSet(CarWheelFLJoint,power)
	      iWheelPowerSet(CarWheelFRJoint,power)
	      iWheelPowerSet(CarWheelRLJoint,power)
	      iWheelPowerSet(CarWheelRRJoint,power)
	   EndIf
	
	   ' apply gravity acceleration To car-body And wheel-bodies
	   iBodyAccelerationApply(CarBody,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   iBodyAccelerationApply(CarWheelFL,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   iBodyAccelerationApply(CarWheelFR,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   iBodyAccelerationApply(CarWheelRL,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   iBodyAccelerationApply(CarWheelRR,_D3DXVECTOR3(0.0,-9.80665,0.0))
	EndMethod
EndType





