' Simultaneous views sample code. How To view the same scene from different
' points of view at the same time.

' A smaller rendering window (camera) is opened on the main, full-display rendering window.

' Interactive ball controls And 3d sound listener can use either
' the main Or the small camera as a reference.

' NOTE: in our code comments we assume you understand the SoundMusic sample code.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global CameraSmall:CAMERA
Global LoadingMessage:SPRITE
Global FontWhite:FONT
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY
Global BallMusic:SOUND

Global BallLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global SmallCameraIsReference:Int

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   ' Create the main, full-display rendering window, as usual
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   ' Create a smaller camera
	   CameraSmall=iCameraCreate(0.30,0.14,0.40,0.18)
	   iCameraFovSet(CameraSmall,40.0)
	   ' Place the New view point 20 meters away from the main one, To the 'right' (+X direction)
	   iCameraLocationSet(CameraSmall,_D3DXVECTOR3(20.0,1.5,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Camera)
	   iSpriteRender(LoadingMessage,False)
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Camera)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   ' Current (initial) ball location is stored in a vector
	   BallLocation = _D3DXVECTOR3(0.0,5.0,0.0)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   iBodyLocationSet(Ball,BallLocation,True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   ' Create a 3d sound source, To test listener location switching...
	   BallMusic=iSoundOggCreate("default_res\\sounds\\monoloop.ogg",True,0)
	   ' ...And place it at the same location as the ball
	   iSoundLocationSet(BallMusic,BallLocation)
	   ' Set the sound volume
	   iSoundVolumeSet(BallMusic,1.0)
	   ' Start playing as a loop
	   iSoundPlay(BallMusic,True)
	
	   ' Set main camera as initial listener reference
	   iListenerCameraSet(Camera)
	   ' Tell this demo we are starting with main camera as reference
	   SmallCameraIsReference = False
	
	   ' Orient both cameras toward the ball
	   iCameraLookAt(CameraSmall,BallLocation,1.0)
	   iCameraLookAt(Camera,BallLocation,1.0)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	EndMethod
	Method _Run()

	   ' Put current ball location in the BallLocation vector variable
	   iBodyLocationCM(Ball,BallLocation)
	
	   ' Update sound source location accordingly (place it at current ball location)
	   iSoundLocationSet(BallMusic,BallLocation)
	
	   ' If [Space] is pressed, re-orient both cameras toward the ball
	   If (iKeyDown(DIK_SPACE))
	      iCameraLookAt(CameraSmall,BallLocation,0.1)
	      iCameraLookAt(Camera,BallLocation,0.1)
	   EndIf
	
	   Local acceleration:D3DXVECTOR3=_D3DXVECTOR3()
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   ' Build an acceleration vector For the ball, based on user key presses
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   ' Tranform the acceleration vector so that it is relative To current reference camera
	   If (SmallCameraIsReference)
	      iCameraOrientation(CameraSmall,cameraorientation)
	      iPrintCentered("Controls and 3d sound listener are relative to the SMALL CAMERA",_D3DXVECTOR2(0.0,11.0),_D3DXVECTOR2(0.5,0.5),0.1,FontWhite,0.0)
	   Else
	      iCameraOrientation(Camera,cameraorientation)
	      iPrintCentered("Controls and 3d sound listener are relative to the MAIN CAMERA",_D3DXVECTOR2(0.0,11.0),_D3DXVECTOR2(0.5,0.5),0.1,FontWhite,0.0)
	   EndIf
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   ' Apply acceleration vector To the ball
	   acceleration.y = 0.0
	   iBodyAccelerationApply(Ball,acceleration)
	
	   ' Manage ball gravity effect
	   If (Not iKeyDown(DIK_LCONTROL))
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   ' Manage reference camera switching
	   If (iKeyClick(DIK_S))
	      SmallCameraIsReference = Not SmallCameraIsReference ' If False gets True And vice-versa
	      ' Update listener reference camera accordingly
	      If (SmallCameraIsReference)
			 iListenerCameraSet(CameraSmall)
	      Else
			 iListenerCameraSet(Camera)
		 EndIf
	   EndIf
	
	   ' Print a text message
	   iPrintCentered("[S] to switch camera, [Space] to track the ball",_D3DXVECTOR2(0.0,10.0),_D3DXVECTOR2(0.5,0.5),0.1,FontWhite,0.0)
	EndMethod
EndType
