' Simulation-consistent sound of a bouncing ball

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY
Global FontWhite:FONT

' Defines Object variable For the ball sound
Global BallSound:SOUND
' Defines dynamic sound-volume variables. See below.
Global BallSoundVol:Float
Global BallSoundVolDuration:Float

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	   iBodyBodyRestitutionSet(BallTerrain,0.75)
	
	   ' In order To trigger the contact sound at proper time, location, volume And frequency,
	   ' we need To know when a collision happens, where it happens, And what is the intensity
	   ' of the force applied To the ball as a consequence of the collision.
	
	   ' So we must enable collision feedback For the BallTerrain collision couple (bodybody).
	   iBodyBodyFeedbackEnable(BallTerrain,-1,0,True)
	
	   ' The source sound should be a continuous And uniform sound that can be played as
	   ' a loop with no glitches.
	
	   ' Best results are achieved by sampling the actual sound emitted by a real world
	   ' source as similar as possible To the one in the simulation.
	
	   ' For example, we have recorded the actual sound of a ball rolling on a board For
	   ' this example. Note that the same looping sound is used To render non-looping
	   ' effects like collisions. See the commented code in the Run() callback For details.
	
	   ' Loads the sound from a file in VorbisOgg compressed format, To be played as a 3d sound.
	   BallSound=iSoundOggCreate("default_res\\sounds\\ball.ogg",True,0)
	   ' Resets dynamic sound-volume variables.
	   BallSoundVol=0.0
	   BallSoundVolDuration=0.0
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	
	   ' Resets the sound source To the same location as the ball
	   iSoundLocationSet(BallSound,_D3DXVECTOR3(0.0,5.0,0.0))
	   ' Resets the sound-volume To zero
	   iSoundVolumeSet(BallSound,0.0)
	   ' Resets the sound-frequency To its normal rate
	   iSoundFrequencySet(BallSound,11025)
	
	   ' Starts playing the sound as a loop.
	   ' Note that the sound is Not audible now, as the volume is zero.
	   iSoundPlay(BallSound,True)
	EndMethod
	Method _Run()

	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,cameratarget)
	   If (iKeyDown(DIK_SPACE)) iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If (Not iKeyDown(DIK_LCONTROL))
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   iPrintCentered("[Ctrl] to bounce, [Space] to track",_D3DXVECTOR2(0.0,-11.5),_D3DXVECTOR2(0.5,0.5),0.1,FontWhite,0.0)
	
	   ' Defines a couple of Local variables (vectors) required by the iBodyBodyFeedback() Function
	   Local location:D3DXVECTOR3=_D3DXVECTOR3(),force:D3DXVECTOR3=_D3DXVECTOR3()
	   ' The ball sound is now constantly playing as a loop, at zero volume.
	   ' The Next line is executed again And again, in the Run() callback, 75 times per second.
	   ' It basically does nothing, unless a collision between the ball And the scenery is
	   ' reported by the iBodyBodyFeedback() function. On a collision, the Function returns
	   ' True, And the If() section is executed.
	   If (iBodyBodyFeedback(BallTerrain,-1,0,location,Null,force,Null,Null,Null))
	      ' If a collision happened during the current dll-loop, this program section is executed.
	
	      ' The vector named 'location' has been filled with the collision location by the
	      ' iBodyBodyFeedback() Function above.
	      ' The vector named 'force' has been filled with the force applied to the ball
	      ' as a consequence of the collision.
	
	      ' The Next line of code basically converts the intensity of the force generated
	      ' by the collision To a value between 0.0f And 1.0f. It puts zero in the
	      ' 'forceintensity' variable when the lenght (intensity) of 'force' is 8.0f.
	      ' It puts 1.0f when the intensity is 40.0f.
	      ' Note that 'forceintensity' is less than zero when the intensity is less than 0.5,
	      ' And it is greater than 1.0f when the intensity is bigger than 200.0f.
	      Local forceintensity:Float=iFloatInterpolate(iVectorLength(force),8.0,40.0,0.0,1.0,False)
	
	      ' The following line checks If 'forceintensity is greater than zero. If so the
	      ' subsequent {...} section is executed.
	      If (forceintensity>0.0)
	         ' The following line computes a sound frequency that is proportional To the
	         ' intensity of the collision. When 'forceintensity' is 0.0, the sound-frequency
	         ' is set To 10000, when it is 1.0f the frequency is set To 10500.
	         ' As a result, when the ball hits the ground harder, the sound tone is slightly higher.
	         Local frequency:Int=iFloatInterpolate(forceintensity,0.0,1.0,10000.0,10500.0,False)
	         ' Applies the frequency To the sound
	         iSoundFrequencySet(BallSound,frequency)
	         ' The following line computes a sound volume that is proportional To the
	         ' intensity of the collision. When 'forceintensity' is 0.0, the sound-volume
	         ' is set To 0.0, when it is 1.0f the volume is set To 20.0f.
	         ' As a result, when the ball hits the ground harder, the sound-volume is slightly
	         ' higher.
	         BallSoundVol=iFloatInterpolate(forceintensity,0.0,1.0,0.0,20.0,False)
	         ' Computed volume must be sustained For some time. Because the sound effect
	         ' is a collision, we set a very Short sustain period. It is still Long enough
	         ' To give the impression of a continuous sound when the collisions are very
	         ' frequent, For example when the ball is rolling. See below For more.
	         BallSoundVolDuration=0.05
	         ' Places the sound source at the same location as the collision point, as
	         ' reported in the 'location' variable by the iBodyBodyFeedback() function above.
	         iSoundLocationSet(BallSound,location)
	      EndIf
	   EndIf
	   ' Updates (decreases) the volume sustain timer.
	   BallSoundVolDuration :- iFrq()
	   ' The following If() section is skipped, unless the sustain timer is
	   ' less than Or equal To zero
	   If (BallSoundVolDuration <= 0.0)
	      ' If the sustain timer is zero Or less, the following line is executed again an again.
	      ' It basically decreases the volume To 0.0, in about 0.05f seconds (sound decay).
	      ' A longer decay would produce a sort of echo.
	      iFloatTendTo(BallSoundVol,0.0,0.05)
	   EndIf
	   ' Applies the current volume To the ball sound. Executed again And again.
	   iSoundVolumeSet(BallSound,BallSoundVol)
	EndMethod
EndType


