' Springy bug example.
' Controls: arrow keys And [Ctrl] To take off.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Type CameramanSprinbyBug

   ' A modified version of the Default cameraman class To
   ' allow For 360 degrees rotations of the actor.

   Field cCamera:CAMERA
   Field cReset:Int
   Field cCameramanBody:BODY
   Field cDistance:Float
   Field cSpring:Float
   Field cDamping:Float
   Field cOrientationTime:Float
   Field cBodyBody:BODYBODY[]
   Field cBodyBodyCount:Int
   Field cCameraOrientation:D3DXQUATERNION=_D3DXQUATERNION()

   Method New()
      cCamera = Null
      cReset = True
      cCameramanBody = Null
      cBodyBody = Null
   EndMethod

   Method create(camera:CAMERA,radius:Float,distance:Float,spring:Float,damping:Float,orientationtime:Float)
      cCamera = camera
      cDistance = distance
      cSpring = spring
      cDamping = damping
      cOrientationTime = orientationtime
      cCameraOrientation = _D3DXQUATERNION(0.0,0.0,0.0,1.0)

      ' create cameraman body And spring
      cCameramanBody=iBodySphereCreate(radius,_D3DXVECTOR3(0.0,0.0,0.0),1.0)
      iBodyDampingSet(cCameramanBody,0.0,cDamping,0.0,0.95)
      Local v:D3DXVECTOR3=_D3DXVECTOR3()
      iCameraLocation(cCamera,v)
      iBodyLocationSet(cCameramanBody,v,True)

      iBodyEnable(cCameramanBody)
   EndMethod

   Method Update(actorlocation:D3DXVECTOR3,targetlocation:D3DXVECTOR3,actor:BODY)
      Local cameramanlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local cameramanorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local endlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local endorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local v:D3DXVECTOR3=_D3DXVECTOR3(),v0:D3DXVECTOR3=_D3DXVECTOR3(),v1:D3DXVECTOR3=_D3DXVECTOR3(),axis:D3DXVECTOR3=_D3DXVECTOR3(),up:D3DXVECTOR3=_D3DXVECTOR3()
      Local q:D3DXQUATERNION=_D3DXQUATERNION()
      Local endangle:Float

      ' store some data we need later
      iCameraOrientation(cCamera,cCameraOrientation)
      iBodyLocation(cCameramanBody,cameramanlocation)

      If (actor)
		 iVectorBodyToWorldRotate(up,_D3DXVECTOR3(0.0,1.0,0.0),actor,0)
      Else 
		up = actorlocation.Minus(targetlocation)
	 EndIf

      ' compute location that satisfies the required distance from the actor
      iVectorLengthSet(v0,cameramanlocation.Minus(actorlocation),cDistance)
      ' compute actorlocation-targetlocation (with cDistance lenght)
      iVectorLengthSet(v1,actorlocation.Minus(targetlocation),cDistance)
      ' compute axis of rotation, To transform v1 To v0
      iVectorCross(axis,v1,v0)
      ' compute current ideal angle
      endangle = iFloatInterpolate(iVectorLength(actorlocation.Minus(targetlocation).Div(cDistance)),0.0,3.0,80.0,20.0,True)
      ' compute quaternion from axis And angle
      iQuaternionFromAxisAngle(q,axis,endangle)
      ' rotate v1 towards v0 To find v
      iVectorRotate(v,v1,q)
      ' find End-location
      endlocation = actorlocation.Plus(v)
      ' ...And from it build a force that pushes the cameraman To it.
      Local l:Float = iVectorLength(cameramanlocation.Minus(endlocation))
      iVectorLengthSet(v,endlocation.Minus(cameramanlocation),l*cSpring)
      Local d:Float = iFloatInterpolate(iVectorLength(cameramanlocation.Minus(endlocation)),1.0,1000.0,cDamping,1.0,True)
      iBodyDampingSet(cCameramanBody,0.0,d,0.0,0.95)
      iBodyAccelerationApply(cCameramanBody,v)

      ' convert orientation time factor To a proper, dll-loop-frequency-dependant, format
      Local orientationtime:Float = cOrientationTime
      If (orientationtime>iFrq())
		 orientationtime = iFrq()/orientationtime
      Else
		 orientationtime=1.0
	 EndIf

      If (cReset)
         cReset = False
         iQuaternionLookAt(endorientation,actorlocation.Minus(endlocation),up)
         iBodyOrientationSet(cCameramanBody,endorientation,True)
         iBodyLocationSet(cCameramanBody,endlocation,True)
         iCameraOrientationSet(cCamera,endorientation)
         iCameraLocationSet(cCamera,endlocation)
      Else
         iQuaternionLookAt(endorientation,actorlocation.Minus(cameramanlocation),up)
         iQuaternionInterpolate(cameramanorientation,cCameraOrientation,endorientation,orientationtime) 
         iCameraOrientationSet(cCamera,cameramanorientation)
         iCameraLocationSet(cCamera,cameramanlocation)
      EndIf
   EndMethod
   Method Reset()
      cReset = True
   EndMethod
   Method Resume()
      Local cameralocation:D3DXVECTOR3=_D3DXVECTOR3()
      iCameraLocation(cCamera,cameralocation)
      iBodyLocationSet(cCameramanBody,cameralocation,True)
      cReset = False
   EndMethod
   Method Destroy()
      cCameramanBody=Null
   EndMethod

   Method Delete()
	 Destroy()
   EndMethod
EndType

Const TOTAL_PLANETS:Int = 2
Const GRAVITY_RADIUS:Float = 3000.0
Const WHEEL_COUNT:Int = 4
Const SHADOW_EXTRUSION:Float = 100.0

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global FontWhite:FONT
Global Prt:String

Global PlanetBody:BODY[TOTAL_PLANETS]
Global PlanetMesh:MESH[TOTAL_PLANETS]
Global PlanetShadowMesh:MESH[TOTAL_PLANETS]
Global PlanetLocation:D3DXVECTOR3[TOTAL_PLANETS]
Global PlanetRadius:Float[TOTAL_PLANETS]
Global LocalPlanet:Int

Global Craft:BODY
Global CraftMesh:MESH
Global CraftShadowMesh:MESH
Global CraftPlanet:BODYBODY[TOTAL_PLANETS]
Global CraftLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global CraftVerticalVector:D3DXVECTOR3=_D3DXVECTOR3()
Global CraftAltitude:Float
Global CraftGravity:Float

Global Wheel:BODY[WHEEL_COUNT]
Global WheelMesh:MESH[WHEEL_COUNT]
Global WheelShadowMesh:MESH[WHEEL_COUNT]
Global WheelPlanet:BODYBODY[WHEEL_COUNT,TOTAL_PLANETS]
Global WheelDock:BODYBODY[WHEEL_COUNT]
Global WheelJoint:WHEEL[WHEEL_COUNT]

Global SuspLocation0:D3DXVECTOR3[WHEEL_COUNT]
For Local z:Int=0 To WHEEL_COUNT-1
	SuspLocation0[z]=_D3DXVECTOR3()
Next

Global SuspLocation1:D3DXVECTOR3[WHEEL_COUNT]
For Local z:Int=0 To WHEEL_COUNT-1
	SuspLocation1[z]=_D3DXVECTOR3()
Next

Global AxleMesh:MESH[2]
Global AxleShadowMesh:MESH[2]
Global RodMesh:MESH[2]
Global RodShadowMesh:MESH[2]
Global RodsMesh:MESH[4]
Global RodsShadowMesh:MESH[4]
Global SuspMesh:MESH[4]
Global SuspShadowMesh:MESH[4]

Global Trail:TRAIL
Global TrailEmitter:TRAILEMITTER[4]
Global TrailEmitterLocation:D3DXVECTOR3[4]

Global CSB:CameramanSprinbyBug=New CameramanSprinbyBug 

Function CraftCreate()

   Local i:Int,j:Int
   Local wheellocation:D3DXVECTOR3[WHEEL_COUNT]
   Local mgLocation:D3DXVECTOR3=_D3DXVECTOR3()

   Local craftBodyMass:Float=2500.0
   Local wheelMass:Float=100.0
   Local sideFriction:Float=10000.0
   Local sideSlip:Float=0.0
   Local rollingWiseFriction:Float=15000.0
   Local rollingWiseSlip:Float=0.0

   CraftLocation = PlanetLocation[LocalPlanet].Plus(_D3DXVECTOR3(0.0,PlanetRadius[LocalPlanet]+400.0,0.0))

   iCameraLocationSet(Camera,CraftLocation.Plus(_D3DXVECTOR3(0.0,10.0,10.0)))
   CSB.Reset()

   Craft=iBodySGCreate("springybug_res\\craftw_.spg",craftBodyMass)
   iBodyLocationSet(Craft,CraftLocation,True)
   iBodyEnable(Craft)
   CraftMesh=iBodyMeshCreate("springybug_res\\craftw.msh",Craft)
   iMeshShow(CraftMesh)
   CraftShadowMesh=iMeshBodyShadowCasterCreate("springybug_res\\craftw_shadow.msh",Craft,SHADOW_EXTRUSION)
   iMeshShow(CraftShadowMesh)
   ' collision
   For i=0 To TOTAL_PLANETS-1
      CraftPlanet[i]=iBodyBodyCreate(Craft,PlanetBody[i])
      iBodyBodyRestitutionSet(CraftPlanet[i],0.5)
      iBodyBodyFrictionSet(CraftPlanet[i],15000.0)
   Next
   ' damping 2, prevent excessive speeds/spins
   iBodyDamping2Set(Craft,100.0,0.9,3000.0,0.9)

   wheellocation[0] = _D3DXVECTOR3(-1.3,-0.75,2.2)
   wheellocation[1] = _D3DXVECTOR3(1.3,-0.75,2.2)
   wheellocation[2] = _D3DXVECTOR3(-1.3,-0.75,-2.2)
   wheellocation[3] = _D3DXVECTOR3(1.3,-0.75,-2.2)
   For i=0 To WHEEL_COUNT-1
      If (i & 1)
         Wheel[i]=iBodySGCreate("springybug_res\\wheelr_.spg",wheelMass)
         WheelMesh[i]=iBodyMeshCreate("springybug_res\\wheelright.msh",Wheel[i])
         WheelShadowMesh[i]=iMeshBodyShadowCasterCreate("springybug_res\\wheelr_shadow.msh",Wheel[i],SHADOW_EXTRUSION)
      Else
         Wheel[i]=iBodySGCreate("springybug_res\\wheell_.spg",wheelMass)
         WheelMesh[i]=iBodyMeshCreate("springybug_res\\wheelleft.msh",Wheel[i])
         WheelShadowMesh[i]=iMeshBodyShadowCasterCreate("springybug_res\\wheell_shadow.msh",Wheel[i],SHADOW_EXTRUSION)
      EndIf

      iBodyEnable(Wheel[i])
      iMeshShow(WheelMesh[i])
      iMeshShow(WheelShadowMesh[i])
      ' collision
      For j=0 To TOTAL_PLANETS-1
         WheelPlanet[i,j]=iBodyBodyCreate(Wheel[i],PlanetBody[j])
         iBodyBodyFrictionModelSet(WheelPlanet[i,j],0)
         iBodyBodyRestitutionSet(WheelPlanet[i,j],0.5)
         iBodyBodySplitFrictionSet(WheelPlanet[i,j],sideFriction,sideSlip,rollingWiseFriction,rollingWiseSlip)
      Next
      ' damping 2
      iBodyDamping2Set(Wheel[i],100.0,0.9,3000.0,0.9)
      ' joint
      WheelJoint[i] = iWheelCreate(Craft,Wheel[i],wheellocation[i],Null)
      iWheelSuspensionSet(WheelJoint[i],30000.0,500.0)
   Next

   For i=0 To 1
      AxleMesh[i]=iMeshCreate("springybug_res\\axle.msh")
      AxleShadowMesh[i]=iMeshShadowCasterCreate("springybug_res\\axle_shadow.msh",SHADOW_EXTRUSION)
      iMeshShow(AxleMesh[i])
      iMeshShow(AxleShadowMesh[i])
      RodMesh[i]=iMeshCreate("springybug_res\\rod.msh")
      RodShadowMesh[i]=iMeshShadowCasterCreate("springybug_res\\rod_shadow.msh",SHADOW_EXTRUSION)
      iMeshShow(RodMesh[i])
      iMeshShow(RodShadowMesh[i])
   Next
   For i=0 To 3
      RodsMesh[i]=iMeshCreate("springybug_res\\rods.msh")
      RodsShadowMesh[i]=iMeshShadowCasterCreate("springybug_res\\rods_shadow.msh",SHADOW_EXTRUSION)
      iMeshShow(RodsMesh[i])
      iMeshShow(RodsShadowMesh[i])
      SuspMesh[i]=iMeshCreate("springybug_res\\susp.msh")
      SuspShadowMesh[i]=iMeshShadowCasterCreate("springybug_res\\susp_shadow.msh",SHADOW_EXTRUSION)
      iMeshShow(SuspMesh[i])
      iMeshShow(SuspShadowMesh[i])
   Next

   Trail=iTrailCreate("default_res\\trail.dds",500)
   iTrailRenderModeSet(Trail,1)
   Local col:D3DXVECTOR3 = _D3DXVECTOR3(0.0,1.0,1.0)
   iTrailColorSet(Trail,col,col)
   iTrailLifeTimeSet(Trail,3.0)
   For i=0 To 3
      If (i = 0) TrailEmitterLocation[i] = _D3DXVECTOR3(-2.0,0.950,0.0)
      If (i = 1) TrailEmitterLocation[i] = _D3DXVECTOR3(-0.55,1.85,-2.0)
      If (i = 2) TrailEmitterLocation[i] = _D3DXVECTOR3(0.55,1.85,-2.0)
      If (i = 3) TrailEmitterLocation[i] = _D3DXVECTOR3(2.0,0.950,0.0)
      TrailEmitter[i]=iTrailEmitterCreate(Trail)
      iTrailEmitterScaleSet(TrailEmitter[i],0.1)
   Next
   iTrailShow(Trail)
EndFunction

Function CraftMeshesPlace()

   Local craftup:D3DXVECTOR3=_D3DXVECTOR3(),vz:D3DXVECTOR3=_D3DXVECTOR3(),vzl:D3DXVECTOR3=_D3DXVECTOR3(),vzr:D3DXVECTOR3=_D3DXVECTOR3(),cfa:D3DXVECTOR3=_D3DXVECTOR3(),cfal:D3DXVECTOR3=_D3DXVECTOR3(),cfar:D3DXVECTOR3=_D3DXVECTOR3(),cra:D3DXVECTOR3=_D3DXVECTOR3(),cral:D3DXVECTOR3=_D3DXVECTOR3(),crar:D3DXVECTOR3=_D3DXVECTOR3()
   Local w:D3DXVECTOR3[4],s:D3DXVECTOR3[4]
   For Local i:Int=0 To 3
	   w[i]=_D3DXVECTOR3()
	   s[i]=_D3DXVECTOR3()
   Next

   Local va:D3DXVECTOR3=_D3DXVECTOR3(),vb:D3DXVECTOR3=_D3DXVECTOR3()
   ' compute up vector
   iVectorBodyToWorldRotate(craftup,_D3DXVECTOR3(0.0,1.0,0.0),Craft,0)
   ' compute car zero location
   iBodyLocation(Craft,vz)
   ' compute zero Left  Right
   iVectorBodyToWorldTransform(vzl,_D3DXVECTOR3(-0.42,0.0,0.0),Craft,0)
   iVectorBodyToWorldTransform(vzr,_D3DXVECTOR3(0.42,0.0,0.0),Craft,0)
   ' compute wheel locations
   iBodyLocation(Wheel[0],w[0])
   iBodyLocation(Wheel[1],w[1])
   iBodyLocation(Wheel[2],w[2])
   iBodyLocation(Wheel[3],w[3])
   ' compute suspension locations
   iVectorBodyToWorldTransform(s[0],_D3DXVECTOR3(-0.4,0.4,1.8),Craft,0)
   iVectorBodyToWorldTransform(s[1],_D3DXVECTOR3(0.4,0.4,1.8),Craft,0)
   iVectorBodyToWorldTransform(s[2],_D3DXVECTOR3(-0.4,0.4,-1.8),Craft,0)
   iVectorBodyToWorldTransform(s[3],_D3DXVECTOR3(0.4,0.4,-1.8),Craft,0)
   ' compute center of front  rear axles
   cfa = w[0].Plus(w[1]).Mult(0.5)
   cra = w[2].Plus(w[3]).Mult(0.5)
   ' compute Left joint of front  rear axles
   cfal = w[0].Plus(w[1].Minus(w[0]).Mult(0.27))
   cral = w[2].Plus(w[3].Minus(w[2]).Mult(0.27))
   ' compute Right joint of front  rear axles
   cfar = w[0].Plus(w[1].Minus(w[0]).Mult(0.73))
   crar = w[2].Plus(w[3].Minus(w[2]).Mult(0.73))
   ' stretch axles
   iMeshStretchToSegment(AxleMesh[0],w[0],w[1],craftup,1.0)
   iMeshStretchToSegment(AxleShadowMesh[0],w[0],w[1],craftup,1.0)
   iMeshStretchToSegment(AxleMesh[1],w[2],w[3],craftup,1.0)
   iMeshStretchToSegment(AxleShadowMesh[1],w[2],w[3],craftup,1.0)
   ' stretch rod
   iMeshStretchToSegment(RodMesh[0],vz,cfa,craftup,2.0)
   iMeshStretchToSegment(RodShadowMesh[0],vz,cfa,craftup,2.0)
   iMeshStretchToSegment(RodMesh[1],vz,cra,craftup,2.0)
   iMeshStretchToSegment(RodShadowMesh[1],vz,cra,craftup,2.0)
   ' stretch rods
   iMeshStretchToSegment(RodsMesh[0],vzl,cfal,craftup,1.0)
   iMeshStretchToSegment(RodsShadowMesh[0],vzl,cfal,craftup,1.0)
   iMeshStretchToSegment(RodsMesh[1],vzl,cral,craftup,1.0)
   iMeshStretchToSegment(RodsShadowMesh[1],vzl,cral,craftup,1.0)
   iMeshStretchToSegment(RodsMesh[2],vzr,cfar,craftup,1.0)
   iMeshStretchToSegment(RodsShadowMesh[2],vzr,cfar,craftup,1.0)
   iMeshStretchToSegment(RodsMesh[3],vzr,crar,craftup,1.0)
   iMeshStretchToSegment(RodsShadowMesh[3],vzr,crar,craftup,1.0)
   ' stretch suspensions
   iMeshStretchToSegment(SuspMesh[0],s[0],w[0],craftup,0.7)
   iMeshStretchToSegment(SuspShadowMesh[0],s[0],w[0],craftup,0.7)
   iMeshStretchToSegment(SuspMesh[1],s[1],w[1],craftup,0.7)
   iMeshStretchToSegment(SuspShadowMesh[1],s[1],w[1],craftup,0.7)
   iMeshStretchToSegment(SuspMesh[2],s[2],w[2],craftup,0.7)
   iMeshStretchToSegment(SuspShadowMesh[2],s[2],w[2],craftup,0.7)
   iMeshStretchToSegment(SuspMesh[3],s[3],w[3],craftup,0.7)
   iMeshStretchToSegment(SuspShadowMesh[3],s[3],w[3],craftup,0.7)
EndFunction

Function CraftStabilize(landed:Int)

   Local i:Int
   Local q:D3DXQUATERNION=_D3DXQUATERNION()
   Local axis:D3DXVECTOR3=_D3DXVECTOR3(),up:D3DXVECTOR3=_D3DXVECTOR3()
   Local angle:Float
   If (landed)
      iVectorBodyToWorldRotate(axis,_D3DXVECTOR3(0.0,1.0,0.0),Craft,0)
      iVectorLengthSet(up,CraftVerticalVector,-1.0)
      Local dot:Float = iVectorDot(up,axis)
      If (dot < 0.75)
         iVectorsAngleToQuaternion(q,axis,CraftVerticalVector.Neg(),1.0)
         iQuaternionToAxisAngle(q,axis,angle)
         iVectorLengthSet(axis,axis,angle*20000.0)
         iBodyTorqueApply(Craft,axis)
      EndIf
      ' damping
      iBodyDampingSet(Craft,50.0,0.95,0.0,1.0)
      For i=0 To WHEEL_COUNT-1
         iBodyDampingSet(Wheel[i],0.0,1.0,0.0,1.0)
      Next
   Else
      iVectorBodyToWorldRotate(axis,_D3DXVECTOR3(0.0,1.0,0.0),Craft,0)
      iVectorLengthSet(up,CraftVerticalVector,-1.0)
      Local dot:Float = iVectorDot(up,axis)
      iVectorsAngleToQuaternion(q,axis,CraftVerticalVector.Neg(),1.0)
      iQuaternionToAxisAngle(q,axis,angle)
      iVectorLengthSet(axis,axis,angle*20000.0)
      iBodyTorqueApply(Craft,axis)
      ' damping
      iBodyDampingSet(Craft,0.0,0.997,0.0,0.99)
      For i=0 To WHEEL_COUNT-1
         iBodyDampingSet(Wheel[i],0.0,0.99,0.0,0.99)
      Next
   EndIf
EndFunction

Function LocalPlanetSet()

   Local i:Int
   Local d:Float
   Local minDist:Float = 1000000000.0
   For i=0 To TOTAL_PLANETS-1
      d = iVectorLength(PlanetLocation[i].Minus(CraftLocation))
      If (d < minDist)
         minDist = d
         LocalPlanet = i
      EndIf
   Next
EndFunction

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Local i:Int
	
	   ' Create the camera
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraFovSet(Camera,60.0)
	
	   ' Set sunlight direction And properties
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.2))
	   iShadowColorSet(_D3DXVECTOR3(0.8,0.8,0.8))
	
	   ' Create And show 'loading...' message sprite
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   ' Create font For stat printing
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   ' Create skybox (3d background)
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	   iSkyBoxShow(SkyBox)
	
	   ' Create planets
	   For i=0 To TOTAL_PLANETS-1
	      If (i = 0)
	         PlanetLocation[i] = _D3DXVECTOR3(0.0,0.0,0.0)
	         PlanetRadius[i] = 12.0
	      ElseIf (i = 1)
	         PlanetLocation[i] = _D3DXVECTOR3(300.0,-300.0,0.0)
	         PlanetRadius[i] = 7.0
		 Else
	         PlanetLocation[i] = _D3DXVECTOR3()
	      EndIf
	      PlanetBody[i]=iBodySphereCreate(PlanetRadius[i],_D3DXVECTOR3(0.0,0.0,0.0),-1.0)
	      PlanetMesh[i]=iBodyMeshCreate("default_res\\planet.x",PlanetBody[i])
	      PlanetShadowMesh[i]=iMeshBodyShadowCasterCreate("default_res\\planet.x",PlanetBody[i],SHADOW_EXTRUSION)
	      If (i = 1) iMeshSecondaryTextureSet(PlanetMesh[i],"springybug_res\\red.bmp",_D3DXVECTOR2(0.0,0.0),PlanetRadius[i]*2.0,0.6)
	      iMeshScaleSet(PlanetMesh[i],PlanetRadius[i],PlanetRadius[i],PlanetRadius[i])
	      iMeshScaleSet(PlanetShadowMesh[i],PlanetRadius[i],PlanetRadius[i],PlanetRadius[i])
	      iBodyLocationSet(PlanetBody[i],PlanetLocation[i],True)
	      iBodyEnable(PlanetBody[i])
	      iMeshShow(PlanetMesh[i])
	      iMeshShow(PlanetShadowMesh[i])
	   Next
	   LocalPlanet = 0
	
	   ' Create the strange craft
	   CraftCreate()
	
	   ' Create the cameraman. Params: camera,radius,distance,spring,damping,orientationtime
	   CSB.Create(Camera,1.0,20.0,7.0,0.2,0.75)
	
	   ' Finished creating the world, hide loading message
	   iSpriteHide(LoadingMessage)
	EndMethod

	Method _Run()

	   Local i:Int
	   Local v:D3DXVECTOR3=_D3DXVECTOR3()
	   Local q:D3DXQUATERNION=_D3DXQUATERNION()
	
	   ' Find the closest planet And make it the current one
	   LocalPlanetSet()
	
	   ' Store current quantities
	   iBodyLocationCM(Craft,CraftLocation)
	   CraftVerticalVector = PlanetLocation[LocalPlanet].Minus(CraftLocation)
	   CraftAltitude = iVectorLength(CraftVerticalVector)- PlanetRadius[LocalPlanet]
	
	   ' Manage cameraman
	   Local vertical:D3DXVECTOR3=_D3DXVECTOR3(),targetLocation:D3DXVECTOR3=_D3DXVECTOR3(),n:D3DXVECTOR3=_D3DXVECTOR3()
	   vertical = PlanetLocation[LocalPlanet].Minus(CraftLocation)
	   If (iBodyScan(PlanetBody[LocalPlanet],CraftLocation,vertical,0.5,targetLocation,n) = -1)
	      targetLocation = PlanetLocation[LocalPlanet]
	   EndIf
	   CSB.cOrientationTime = iFloatInterpolate(CraftAltitude,0.0,200.0,0.1,1.0,True)
	   CSB.cSpring = iFloatInterpolate(CraftAltitude,0.0,200.0,500.0,50.0,True)
	   CSB.Update(CraftLocation,targetLocation,Null)
	
	   ' Manage visible craft features (red springs, etc)
	   CraftMeshesPlace()
	
	   ' ********************************
	   ' Manage rockets, wheels, thruster
	
	   Local landed:Int = False
	   If (iVectorLength(targetLocation.Minus(CraftLocation)) < 3.0) landed = True
	
	   Global rocketsPower:Float = 0.0
	   Global steerAngle:Float = 0.0
	   Global wheelSpin:Float = 400.0
	   Global wspin:Float = 0.0
	   Global wheelPower:Float = 4000.0
	
	   ' rockets control
	   Local rocketsForce:D3DXVECTOR3=_D3DXVECTOR3()
	   If (iKeyDown(DIK_LCONTROL) Or iKeyDown(DIK_LCONTROL))
	      iFloatTendTo(rocketsPower,-100000.0,1.0)
	   Else
		 iFloatTendTo(rocketsPower,0.0,0.25)
	   EndIf
	   iVectorLengthSet(rocketsForce,CraftVerticalVector,rocketsPower)
	   iBodyForceApply(Craft,rocketsForce,Null)
	
	   If (landed)
	      ' wheels' steer angle control
	      If (iKeyDown(DIK_LEFT))
	         iFloatTendTo(steerAngle,20.0,0.3)
	      ElseIf (iKeyDown(DIK_RIGHT))
	         iFloatTendTo(steerAngle,-20.0,0.3)
	      Else
		    iFloatTendTo(steerAngle,0.0,0.2)
	      EndIf
	
	      ' wheels' power control
	      If (iKeyDown(DIK_UP))
	         iFloatTendTo(wspin,-wheelSpin,0.4)
	      ElseIf (iKeyDown(DIK_DOWN))
	         iFloatTendTo(wspin,wheelSpin,0.4)
	      Else
		    iFloatTendTo(wspin,0.0,0.1)
		 EndIf
	   Else
	      ' we are flying, manage rocket forces, etc
	      Local thrusterForce:D3DXVECTOR3=_D3DXVECTOR3()
	      If (iKeyDown(DIK_LEFT))
	         iVectorBodyToWorldRotate(thrusterForce,_D3DXVECTOR3(10000.0,0.0,0.0),Craft,0)
	         iBodyForceApply(Craft,thrusterForce,_D3DXVECTOR3(0.0,0.0,-2.0))
	      EndIf
	      If (iKeyDown(DIK_RIGHT))
	         iVectorBodyToWorldRotate(thrusterForce,_D3DXVECTOR3(-10000.0,0.0,0.0),Craft,0)
	         iBodyForceApply(Craft,thrusterForce,_D3DXVECTOR3(0.0,0.0,-2.0))
	      EndIf
	      If (iKeyDown(DIK_UP))
	         iVectorBodyToWorldRotate(thrusterForce,_D3DXVECTOR3(0.0,0.0,35000.0),Craft,0)
	         iBodyForceApply(Craft,thrusterForce,Null)
	      EndIf
	      If (iKeyDown(DIK_DOWN))
	         iVectorBodyToWorldRotate(thrusterForce,_D3DXVECTOR3(0.0,0.0,-35000.0),Craft,0)
	         iBodyForceApply(Craft,thrusterForce,Null)
	      EndIf
	      iFloatTendTo(wspin,0.0,0.1)
	      iFloatTendTo(steerAngle,0.0,0.2)
	   EndIf
	
	   ' apply steerangle, power,thruster
	   For i=0 To 1
		iWheelSteeringAngleSet(WheelJoint[i],steerAngle)
	   Next
	   For i=2 To 3
		iWheelSteeringAngleSet(WheelJoint[i],-steerAngle)
	   Next
	   For i=0 To WHEEL_COUNT-1
	      iWheelSpinSet(WheelJoint[i],wspin)
	      Local power:Float = iFloatInterpolate(iFloatAbs(wspin),0.0,wheelSpin,wheelPower*0.5,wheelPower,False)
	      iWheelPowerSet(WheelJoint[i],power)
	   Next
	
	   ' ********************************
	
	   ' Manage trails
	   Local location:D3DXVECTOR3=_D3DXVECTOR3()
	   Local velocity:D3DXVECTOR3=_D3DXVECTOR3()
	   Local orientation:D3DXQUATERNION=_D3DXQUATERNION()
	   Local alpha:Float
	   For i=0 To 3
	      iVectorBodyToWorldTransform(location,TrailEmitterLocation[i],Craft,0)
	      iTrailEmitterLocationSet(TrailEmitter[i],location)
	      iBodyPointVelocity(Craft,TrailEmitterLocation[i],velocity)
	      iQuaternionLookAt(orientation,velocity,_D3DXVECTOR3(0.0,1.0,0.0))
	      iTrailEmitterOrientationSet(TrailEmitter[i],orientation)
	      alpha = iFloatInterpolate(iVectorLength(velocity),10.0,50.0,0.0,1.0,True)
	      iTrailEmitterEmit(TrailEmitter[i],14.0,alpha)
	   Next
	
	   ' Manage gravity
	   If (CraftAltitude < GRAVITY_RADIUS)
	      Local gravity:D3DXVECTOR3=_D3DXVECTOR3()
	      CraftGravity = iFloatInterpolate(CraftAltitude,GRAVITY_RADIUS,0.0,0.0,9.80665,True)
	      iVectorLengthSet(gravity,CraftVerticalVector,CraftGravity)
	      iBodyAccelerationApply(Craft,gravity)
	      For i=0 To WHEEL_COUNT-1
	         iBodyAccelerationApply(Wheel[i],gravity)
	      Next
	   EndIf
	
	   ' Keep the craft straight For current planet
	   CraftStabilize(landed)
	
	   ' Print stats
	   Prt="ALTITUDE: "+String.FromFloat(CraftAltitude)[0..5]
	   iPrintCentered(Prt,_D3DXVECTOR2(0.0,11.5),_D3DXVECTOR2(0.5,0.35),0.1,FontWhite,0.0)
	   Prt="GRAVITY: "+String.FromFloat(CraftGravity/9.80665)[0..4]
	   iPrintCentered(Prt,_D3DXVECTOR2(0.0,11.0),_D3DXVECTOR2(0.5,0.35),0.1,FontWhite,0.0)
	   If (Not LocalPlanet)
		 iPrintCentered("LOCAL PLANET: EARTH",_D3DXVECTOR2(0.0,10.5),_D3DXVECTOR2(0.5,0.35),0.1,FontWhite,0.0)
	   Else
		 iPrintCentered("LOCAL PLANET: MARS",_D3DXVECTOR2(0.0,10.5),_D3DXVECTOR2(0.5,0.35),0.1,FontWhite,0.0)
	   EndIf
	EndMethod

	Method _Exit()
	   ' Remove cameraman
	   CSB=Null
	EndMethod
EndType