' Flick-based texture animation demo.

' A sequence of ten .jpg images (a video clip) replaces the Default ball's
' texture To perform a flick-based surface animation.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY

Global Texture:TEXTURE[10]
Global AnimTimer:Float
Global AnimFlick:Int

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	
	   ' Define a temporary filepath String
	   Local filename:String
	   ' Create the ten textures forming our animation
	   For Local i:Int=0 To 9
	      ' Build path+filename For the current image To load.
	      ' Basically, '%02d' is replaced with 00 when i=0, with 01 when i=1, etc.
	      filename="default_res\\textureanim\\tb0"+i+".jpg"
	      ' Create the current texture by loading the current image file
	      Texture[i] = iTextureCreate(filename)
	   Next
	   ' Initialize animation timer And counter
	   AnimTimer = 0.0
	   AnimFlick = 0
	   ' Ensure we start with the first image of the sequence assigned To the ball texture
	   iMeshTextureSwap(BallMesh,Texture[AnimFlick])
	EndMethod

	Method _Run()
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   ' Only change image If timer is zero Or less, that is every 20th of second
	   If (AnimTimer<=0.0)
	      ' The timer is zero Or less, reset it
	      AnimTimer=0.05
	      ' Swap back the original mesh texture (also restoring the
	      ' original sequence of free textures)
	      iMeshTextureSwap(BallMesh,Texture[AnimFlick])
	      ' Update flick counter
	      AnimFlick:+1
	      If (AnimFlick>=10) AnimFlick=0
	      ' Assign the New image of the sequence To the mesh
	      iMeshTextureSwap(BallMesh,Texture[AnimFlick])
	   EndIf
	   ' Update the timer
	   AnimTimer :- iFrq()
	EndMethod
EndType




