'Tinted glass demo.

'NOTE: in our code comments we assume you understand the GlassEffect sample code.

'THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY
Global Envmap:ENVMAP
Global TintedGlassMesh:MESH
Global TintedGlassFrameMesh:MESH
Global LookAtPoint:D3DXVECTOR3=_D3DXVECTOR3()
Global LookAtPointAngle:Float

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(-5.0,3.0,10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	   iShadowColorSet(_D3DXVECTOR3(0.85,0.85,0.85))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,15.0,-4.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,50.0)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,2.0)
	
	   'Create a window. It is a two part mesh: a glass And a frame.
	   'The glass mesh is one-sided. Actually, you don't need a solid to
	   'perform a solid-glass (refractive) effect.
	   TintedGlassMesh=iMeshCreate("default_res\\tintedglass.x")
	   iMeshLocationSet(TintedGlassMesh,_D3DXVECTOR3(0.0,3.0,0.0))
	   TintedGlassFrameMesh=iMeshCreate("default_res\\tintedglassframe.x")
	   iMeshLocationSet(TintedGlassFrameMesh,_D3DXVECTOR3(0.0,3.0,0.0))
	
	   'Enable envmap functionality For the glass mesh
	   Envmap=iEnvMapCreate("default_res\\envmap.dds")
	   iMeshEnvMapReceiverAdd(TintedGlassMesh,Envmap)
	   'Set degree of transparency. The less transparent, the more tinted.
	   iMeshEnvMapBlendSet(TintedGlassMesh,0.73)
	   'The Next line actually enables refraction.
	   iMeshEnvMapGlassSet(TintedGlassMesh,0.25)
	
	   'Set refresh rate For refracted image
	   iEnvMapUpdateRateSet(Envmap,60.0)
	
	   'Set meshes we want To refract (be visible behind the glass)
	   iSkyBoxEnvMapCasterAdd(SkyBox)
	   iMeshEnvMapCasterAdd(TerrainMesh)
	   iMeshEnvMapCasterAdd(BallMesh)
	
	   'Set Default quantities For glass animation (see Run() callback)
	   LookAtPoint = _D3DXVECTOR3(0.0,15.0,-50.0)
	   LookAtPointAngle = 0.0
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	   iMeshShow(TintedGlassMesh)
	   iMeshShow(TintedGlassFrameMesh)
	EndMethod
	
	Method _Run()
	   Local ballLocation:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,ballLocation)
	   iCameraLookAt(Camera,ballLocation,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraOrientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraOrientation)
	   iVectorRotate(acceleration,acceleration,cameraOrientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   'Animate the glass
	   Local q:D3DXQUATERNION=_D3DXQUATERNION()
	   Local v:D3DXVECTOR3=_D3DXVECTOR3()
	   'compute current look-at point location
	   iQuaternionFromAxisAngle(q,_D3DXVECTOR3(0.0,0.0,1.0),LookAtPointAngle)
	   iVectorRotate(v,LookAtPoint,q)
	   'compute glass orientation using look-at point as a reference
	   iQuaternionLookAt(q,v,Null)
	   'apply orientation To glass
	   iMeshOrientationSet(TintedGlassMesh,q)
	   iMeshOrientationSet(TintedGlassFrameMesh,q)
	   'update rotation angle about Z axis For look-at point. The look-at point
	   'rotate in a 15 meter radius circle about Z axis, at Z = -50.0f
	   LookAtPointAngle :+ 200.0*iFrq()
	
	   'Set the enviroment map focus at camera location
	   iCameraLocation(Camera,v)
	   iEnvMapFocusSet(Envmap,v)
	
	   'Hide the ball mesh in the environment map image, If the ball
	   'is Not behind the glass plane. Which means that it will Not
	   'be visible in the refracted image when it is between eye And glass.
	   Local glassSurfaceNormal:D3DXVECTOR3=_D3DXVECTOR3()
	   Local glassLocation:D3DXVECTOR3=_D3DXVECTOR3()
	   iMeshLocation(TintedGlassMesh,glassLocation)
	   iVectorMeshToWorldRotate(glassSurfaceNormal,_D3DXVECTOR3(0.0,0.0,-1.0),TintedGlassMesh)
	   iMeshEnvMapCasterAdd(BallMesh)
	   If (iVectorDot(glassSurfaceNormal,(ballLocation.Minus(glassLocation))) > 0.0) iMeshEnvMapCasterRemove(BallMesh)
	EndMethod
EndType
