' True mirror demo

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global Room:BODY
Global RoomMesh:MESH
Global RoomMirrorMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallRoom:BODYBODY

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   ' Create room body. It is basically a box with all triangles facing inward
	   Room=iBodyCreate("default_res\\mirrorroom_4.71.ply")
	   ' Attach a visible mesh To it. A texture mapped box, with all
	   ' sides facing inward. But it has no floor.
	   RoomMesh=iBodyMeshCreate("default_res\\mirrorroom.x",Room)
	
	   ' The Floor must be a different mesh so we can make it a 'mirror'
	   RoomMirrorMesh=iBodyMeshCreate("default_res\\mirrorfloor.x",Room)
	   ' Change To a custom alpha texture. The mirror mesh must be at least
	   ' partially transparent. The mirror trick is that, before rendering the
	   ' normal scene, a 'mirrored' version of it is rendered, but only within
	   ' the bounds of the mirror-mesh. It is basically rendered 'behind' the
	   ' mirror mesh, And can be seen 'through' it. That's why the mirror mesh
	   ' must be at least partially transparent.
	   iMeshAlphaTextureChange(RoomMirrorMesh,"default_res\\mirrorfloor_a.jpg")
	   ' Tell the engine that the Floor is the mirror-mesh
	   iMeshMirrorMake(RoomMirrorMesh,True)
	   ' Finally set the mirror plane so that it matches the mirror mesh.
	   ' The mirror plane is used by the engine as a reference when 'mirroring' the
	   ' whole scene, before rendering it into the mirror mesh.
	   iMirrorPlaneSet(_D3DXVECTOR3(0.0,0.0,0.0),_D3DXVECTOR3(0.0,1.0,0.0))
	
	   ' We want all shadows To be reflected into the mirror. This effect
	   ' tends To worsen a little the framerate, but it is so cool :-)
	   iMirrorShadowsSet(True)
	
	   ' Create a ball To bounce around in the room
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   BallRoom=iBodyBodyCreate(Ball,Room)
	   iBodyBodyFrictionSet(BallRoom,10.0)
	
	   iBodyEnable(Room)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iMeshShow(RoomMesh)
	   iMeshShow(RoomMirrorMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	EndMethod
	Method _Run()
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	EndMethod
EndType

