
#include "windows.h"

#define _3IMPACT_EXTERN
#define WIN32DLL_API
#define CALLMODE_API

#include "dll3impact.h"

extern "C"
{
	#include <math.h>
}

// -----------------------------------------------------------------
// PreRender support
// -----------------------------------------------------------------

void (*PreRenderCallbackPtr)(CAMERA*)=0;

void __stdcall PreRenderCallback(CAMERA* cam)
{
     if (PreRenderCallbackPtr)
		 PreRenderCallbackPtr(cam);
}

extern "C" void bind_PreRenderCallback(void (*PreRenderBinder)(CAMERA*))
{
	PreRenderCallbackPtr=PreRenderBinder;
	i3ImpactPreRenderCallbackSet(PreRenderCallback);
}


// -----------------------------------------------------------------
// window creation
// -----------------------------------------------------------------

LRESULT CALLBACK WndProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
   //Process messages for rendering window (windowed mode) if a
   //custom window is provided. See WinMain() below for details.
   if (uMsg == WM_PAINT)
   {
      if (i3ImpactIsReady())
      {
         //Clear rendering window where necessary (for example at startup)
         iScreenWipe(0,0,0);
      }
   }
   if (uMsg == WM_CLOSE)
   {
      //Process close messages
      i3ImpactIsRenderingSet(FALSE);
      return 0;
   }
   return DefWindowProc(hWnd,uMsg,wParam,lParam);
}
LRESULT CALLBACK FullScreenWndProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
   //Process messages for rendering window (fullscreen mode) if a
   //custom window is provided. See WinMain() below for details.
   if (uMsg == WM_PAINT)
   {
      if (i3ImpactIsReady())
      {
         //Clear full screen display where necessary (for example at startup)
         iScreenWipe(0,0,0);
      }
   }
   if (uMsg == WM_CLOSE)
   {
      //Ignore close messages
      return 0;
   }
   if (uMsg == WM_SETCURSOR)
   {
      //Turn off Windows cursor
      SetCursor(NULL);
      return TRUE;
   }
   return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

//int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)

extern "C" HWND i3ImpactCreateWindow(int width,int height,char* wName,DWORD style=WS_POPUP|WS_CAPTION)
{
   //This is a simple application showing how to call main 3Impact game engine DLL functions to
   //initialize the engine, manage main processing loop and provide custom rendering windows.
   //Please refer to '3Impact C++ Functions Reference' document ('Main Functions' section) for
   //details on their usage.

   HWND hWnd = NULL;
   HINSTANCE hInstance=GetModuleHandleA(0);

   //Create rendering window (windowed mode).
   //If you comment out (remove) this section, a NULL window is passed to the engine.
   //As a consequence, the engine will use its own default rendering window.
   RECT rc = {0,0,width-1,height-1};
   WNDCLASS wndClass = {0,WndProc,0,0,hInstance,0L,NULL,(HBRUSH)GetStockObject(BLACK_BRUSH),NULL,wName};
   RegisterClass(&wndClass);
   hWnd = CreateWindow(wName,wName,style,rc.left,rc.top,(rc.right-rc.left),(rc.bottom-rc.top),0L,0L,hInstance,0L);

   return hWnd;
}

extern "C" HWND i3ImpactCreateFullscreenWindow(HWND hWnd,char* wFsName,DWORD style=WS_POPUP)
{
   //This is a simple application showing how to call main 3Impact game engine DLL functions to
   //initialize the engine, manage main processing loop and provide custom rendering windows.
   //Please refer to '3Impact C++ Functions Reference' document ('Main Functions' section) for
   //details on their usage.

   HWND hFullScreen = NULL;
   HINSTANCE hInstance=GetModuleHandleA(0);

   //Create rendering window (fullscreen mode)
   WNDCLASS wndClassFS = {CS_HREDRAW|CS_VREDRAW,FullScreenWndProc,0,0,hInstance,NULL,NULL,(HBRUSH)GetStockObject(BLACK_BRUSH),NULL,wFsName};
   RegisterClass(&wndClassFS);
   hFullScreen = CreateWindow(wFsName,NULL,style,0,0,10,10,hWnd,0L,hInstance,0L);

   return hFullScreen;
}

extern "C" void i3ImpactDestroyWindow(HWND hWnd)
{
	if (hWnd)
		DestroyWindow(hWnd);
}
