Global _g_3IEngine:_3IEngine

' create an initial instance of the engine.  this guarantees that if no special
' features of the mod are used then the mod will still function properly.
New _3IEngine

Type _3IEngine

	Field network:_3INETWORK=Null
	Field math:_3IMATH=Null
	
	Method New()
		' set the global engine variable
		_g_3IEngine=Self
		
		' create an instance of the network type
		network=generateNetwork()
		
		' create an instance of the math type
		math=generateMath()
		
		bind_PreRenderCallback(_3IEngine.PreRenderCallback)
	EndMethod
	
	Function PreRenderCallback(camptr:Byte Ptr)
		Local cam:CAMERA=_g_3IEngine.generateCamera()
		If cam<>Null 
			cam.objptr=camptr
			_g_3IEngine._PreRender(cam)		
		EndIf
	EndFunction
	
	Method _PreRender(cam:CAMERA)
	EndMethod
	Method _Init()
	EndMethod
	Method _Exit()
	EndMethod
	Method _Run()
	EndMethod

	Method Start()
		' The rendering loop
		While (i3ImpactIsRendering())
			iProcessMessages()
               ' process And render a frame.
               If (i3ImpactIsActive() And i3ImpactIsReady())
               	i3ImpactMain()

               	If (i3ImpactIsExecuting())
                    	If (i3ImpactIsInitializing())
	                        i3ImpactInitBegin()
	                        _Init()
	                        i3ImpactInitEnd()
	                        If (i3ImpactErrorFlag()) Then Exit
                     	EndIf
				EndIf

				If Not i3ImpactIsRendering() Then Continue

                    If Not i3ImpactIsSkippingRunPhase()
                    	i3ImpactRunBegin()
                    	_Run()
                    	If (i3ImpactErrorFlag()) Then Exit
                    	i3ImpactRunEnd()
                    EndIf

                    If Not i3ImpactIsSkippingRenderPhase()
	                    i3ImpactRender()
               	EndIf
               EndIf
		Wend
		i3ImpactExitBegin()
		_Exit()
		i3ImpactExitEnd()
	EndMethod	

	Method CreateWindow:Byte Ptr(width:Int,height:Int,wName:String,style:Int=WS_POPUP|WS_CAPTION)
		Return _i3ImpactCreateWindow(width,height,wName,style)
	EndMethod
		
	Method CreateFullscreenWindow:Byte Ptr(hWnd:Byte Ptr,wFsName:String,style:Int=WS_POPUP)
		Return _i3ImpactCreateFullscreenWindow(hWnd,wFsName,style)
	EndMethod
	
	Method generateNetwork:_3INETWORK()
		Return New _3INETWORK
	EndMethod
	
	Method generateMath:_3IMATH()
		Return New _3IMATH
	EndMethod

	Method generateQuat:D3DXQUATERNION()
		Return New D3DXQUATERNION
	EndMethod

	Method _D3DXQUATERNION:D3DXQUATERNION(x:Float=0,y:Float=0,z:Float=0,w:Float=1.0)
		Local temp:D3DXQUATERNION=_g_3IEngine.generateQuat()
		temp.x=x
		temp.y=y
		temp.z=z
		temp.w=w
		Return temp
	EndMethod
				
	Method generatePlane:D3DXPLANE()
		Return New D3DXPLANE
	EndMethod

	Method _D3DXPLANE:D3DXPLANE(a:Float,b:Float,c:Float,d:Float)
		Local temp:D3DXPLANE=_g_3IEngine.generatePlane()
		temp.a=a
		temp.b=b
		temp.c=c
		temp.d=d
		Return temp
	EndMethod

	Method generateVector2:D3DXVECTOR2()
		Return New D3DXVECTOR2
	EndMethod
			
	Method _D3DXVECTOR2:D3DXVECTOR2(x:Float=0,y:Float=0)
		Local temp:D3DXVECTOR2=_g_3IEngine.generateVector2()
		temp.x=x
		temp.y=y
		Return temp
	EndMethod

	Method generateVector3:D3DXVECTOR3()
		Return New D3DXVECTOR3
	EndMethod

	Method _D3DXVECTOR3:D3DXVECTOR3(x:Float=0,y:Float=0,z:Float=0)
		Local temp:D3DXVECTOR3=_g_3IEngine.generateVector3()
		temp.x=x
		temp.y=y
		temp.z=z
		Return temp
	EndMethod
	
	Method generateVector4:D3DXVECTOR4()
		Return New D3DXVECTOR4
	EndMethod

	Method _D3DXVECTOR4:D3DXVECTOR4(x:Float=0,y:Float=0,z:Float=0,w:Float=0)
		Local temp:D3DXVECTOR4=_g_3IEngine.generateVector4()
		temp.x=x
		temp.y=y
		temp.z=z
		temp.w=w
		Return temp
	EndMethod

	Method generateMatrix:D3DXMATRIX()
		Return New D3DXMATRIX
	EndMethod

	Method _D3DXMATRIX:D3DXMATRIX(matrix:Float[])
		Local temp:D3DXMATRIX=_g_3IEngine.generateMatrix()
		temp._11=matrix[0]
		temp._12=matrix[1]
		temp._13=matrix[2]
		temp._14=matrix[3]
		temp._21=matrix[4]
		temp._22=matrix[5]
		temp._23=matrix[6]
		temp._24=matrix[7]
		temp._31=matrix[8]
		temp._32=matrix[9]
		temp._33=matrix[10]
		temp._34=matrix[11]
		temp._41=matrix[12]
		temp._42=matrix[13]
		temp._43=matrix[14]
		temp._44=matrix[16]
		
		Return temp
	EndMethod
		
	Method generateMeshVertexBuffer:MESHVERTEXBUFFER()
		Return New MESHVERTEXBUFFER
	EndMethod

	Method _MESHVERTEXBUFFER:MESHVERTEXBUFFER()
		Return New MESHVERTEXBUFFER
	EndMethod

	Method generateMeshFaceBuffer:MESHFACEBUFFER()
		Return New MESHFACEBUFFER
	EndMethod

	Method _MESHFACEBUFFER:MESHFACEBUFFER()
		Return New MESHFACEBUFFER
	EndMethod

	Method generateCamera:CAMERA()
		Return New CAMERA
	EndMethod

	Method CameraCreate:CAMERA(top:Float,left_:Float,width:Float,height:Float,generateFunc:Byte Ptr=Null)
		Local retval:CAMERA
		If generateFunc<>Null
			Local genFunc:CAMERA()=generateFunc
			retval=genFunc()
		Else
			retval=generateCamera()
		EndIf
		retval.objptr=_iCameraCreate(top,left_,width,height)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod

	Method generateCameraEffect:CAMERAEFFECT()
		Return New CAMERAEFFECT
	EndMethod

	Method CameraEffectCreate:CAMERAEFFECT(shader_filename:String, generateFunc:Byte Ptr=Null)
		Local retval:CAMERAEFFECT
		If generateFunc<>Null
			Local genFunc:CAMERAEFFECT()=generateFunc
			retval=genFunc()
		Else
			retval=generateCameraEffect()
		EndIf
		retval.objptr=_iCameraEffectCreate(shader_filename)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod

	Method generateSkyBox:SKYBOX()
		Return New SKYBOX
	EndMethod

	Method SkyBoxCreate:SKYBOX(sky_filename:String,generateFunc:Byte Ptr=Null)
		Local retval:SKYBOX
		If generateFunc<>Null
			Local genFunc:SKYBOX()=generateFunc
			retval=genFunc()
		Else
			retval=generateSkyBox()
		EndIf
		retval.objptr=_iSkyBoxCreate(sky_filename)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod

	Method generateBody:BODY()
		Return New BODY
	EndMethod

	Method BodyCreate:BODY(ply_filename:String,generateFunc:Byte Ptr=Null)
		Local retval:BODY
		If generateFunc<>Null
			Local genFunc:BODY()=generateFunc
			retval=genFunc()
		Else
			retval=generateBody()
		EndIf
		retval.objptr=_iBodyCreate(ply_filename)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod

	Method BodyCreateFromMemory:BODY(ply_file_ptr:Byte Ptr,ply_memsize:Int,pol_file_ptr:Byte Ptr,pol_memsize:Int,generateFunc:Byte Ptr=Null)
		Local retval:BODY
		If generateFunc<>Null
			Local genFunc:BODY()=generateFunc
			retval=genFunc()
		Else
			retval=generateBody()
		EndIf
		retval.objptr=_iBodyCreateFromMemory(ply_file_ptr,ply_memsize,pol_file_ptr,pol_memsize)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method BodySGCreate:BODY(spg_filename:String,total_mass:Float,generateFunc:Byte Ptr=Null)
		Local retval:BODY
		If generateFunc<>Null
			Local genFunc:BODY()=generateFunc
			retval=genFunc()
		Else
			retval=generateBody()
		EndIf
		retval.objptr=_iBodySGCreate(spg_filename,total_mass)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method BodySGCreateFromMemory:BODY(spg_file_ptr:Byte Ptr,spg_memsize:Int,total_mass:Float,generateFunc:Byte Ptr=Null)
		Local retval:BODY
		If generateFunc<>Null
			Local genFunc:BODY()=generateFunc
			retval=genFunc()
		Else
			retval=generateBody()
		EndIf
		retval.objptr=_iBodySGCreateFromMemory(spg_file_ptr,spg_memsize,total_mass)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method BodySphereCreate:BODY(radius:Float,offset:D3DXVECTOR3,total_mass:Float,generateFunc:Byte Ptr=Null)
		Local retval:BODY
		If generateFunc<>Null
			Local genFunc:BODY()=generateFunc
			retval=genFunc()
		Else
			retval=generateBody()
		EndIf
		retval.objptr=_iBodySphereCreate(radius,offset.param(),total_mass)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod

	Method generateBodyBody:BODYBODY()
		Return New BODYBODY
	EndMethod
		
	Method BodyBodyCreate:BODYBODY(firstBody:BODY,secondBody:BODY,generateFunc:Byte Ptr=Null)
		Local retval:BODYBODY
		If generateFunc<>Null
			Local genFunc:BODYBODY()=generateFunc
			retval=genFunc()
		Else
			retval=generateBodyBody()
		EndIf
		retval.objptr=_iBodyBodyCreate(firstBody.objptr,secondBody.objptr)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod

	Method generateJoint:JOINT()
		Return New JOINT
	EndMethod

	Method JointBallCreate:JOINT(firstBody:BODY,secondBody:BODY,joint:D3DXVECTOR3,generateFunc:Byte Ptr=Null)
		Local retval:JOINT
		If generateFunc<>Null
			Local genFunc:JOINT()=generateFunc
			retval=genFunc()
		Else
			retval=generateJoint()
		EndIf
		retval.objptr=_iJointBallCreate(firstBody.objptr,secondBody.objptr,joint.param())
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method JointHingeCreate:JOINT(firstBody:BODY,secondBody:BODY,jointLocation:D3DXVECTOR3,jointDirection:D3DXVECTOR3,generateFunc:Byte Ptr=Null)
		Local retval:JOINT
		If generateFunc<>Null
			Local genFunc:JOINT()=generateFunc
			retval=genFunc()
		Else
			retval=generateJoint()
		EndIf
		retval.objptr=_iJointHingeCreate(firstBody.objptr,secondBody.objptr,jointLocation.param(),jointDirection.param())
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method JointSliderCreate:JOINT(firstBody:BODY,secondBody:BODY,jointDirection:D3DXVECTOR3,generateFunc:Byte Ptr=Null)
		Local retval:JOINT
		If generateFunc<>Null
			Local genFunc:JOINT()=generateFunc
			retval=genFunc()
		Else
			retval=generateJoint()
		EndIf
		retval.objptr=_iJointSliderCreate(firstBody.objptr,secondBody.objptr,jointDirection.param())
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
		
	Method generateWheel:WHEEL()
		Return New WHEEL
	EndMethod

	Method WheelCreate:WHEEL(mainBody:BODY,wheel:BODY,mainBodyLocation:D3DXVECTOR3,wheelOrientation:D3DXQUATERNION,generateFunc:Byte Ptr=Null)
		Local retval:WHEEL
		If generateFunc<>Null
			Local genFunc:WHEEL()=generateFunc
			retval=genFunc()
		Else
			retval=generateWheel()
		EndIf
		Local pWheelOrientation:Byte Ptr=Null
		If wheelOrientation<>Null Then pWheelOrientation=wheelOrientation.param()
		
		retval.objptr=_iWheelCreate(mainBody.objptr,wheel.objptr,mainBodyLocation.param(),pWheelOrientation)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
		
	Method generateMesh:MESH()
		Return New MESH
	EndMethod

	Method MeshCreate:MESH(x_filename:String,generateFunc:Byte Ptr=Null)
		Local retval:MESH
		If generateFunc<>Null
			Local genFunc:MESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateMesh()
		EndIf
		retval.objptr=_iMeshCreate(x_filename)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method MeshCreateFromMemory:MESH(x_file_memptr:Byte Ptr,x_memsize:Int,texture_file_memptr:Byte Ptr,texture_memsize:Int,texture_id:Int,generateFunc:Byte Ptr=Null)
		Local retval:MESH
		If generateFunc<>Null
			Local genFunc:MESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateMesh()
		EndIf
		retval.objptr=_iMeshCreateFromMemory(x_file_memptr,x_memsize,texture_file_memptr,texture_memsize,texture_id)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method MeshPartCreate:MESH(x_filename:String,mapped_with_texture:Int,generateFunc:Byte Ptr=Null)
		Local retval:MESH
		If generateFunc<>Null
			Local genFunc:MESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateMesh()
		EndIf
		retval.objptr=_iMeshPartCreate(x_filename,mapped_with_texture)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method MeshPartCreateFromMemory:MESH(x_file_memptr:Byte Ptr,x_memsize:Int,texture_file_memptr:Byte Ptr,texture_memsize:Int,texture_id:Int,mapped_with_texture:Int,generateFunc:Byte Ptr=Null)
		Local retval:MESH
		If generateFunc<>Null
			Local genFunc:MESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateMesh()
		EndIf
		retval.objptr=_iMeshPartCreateFromMemory(x_file_memptr,x_memsize,texture_file_memptr,texture_memsize,texture_id,mapped_with_texture)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod

	Method MeshBumpCreate:MESH(x_filename:String,texture_filename:String,bConvertToNormalMap:Int,generateFunc:Byte Ptr=Null)
		Local retval:MESH
		If generateFunc<>Null
			Local genFunc:MESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateMesh()
		EndIf
		retval.objptr=_iMeshBumpCreate(x_filename,texture_filename,bConvertToNormalMap)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method MeshBumpCreateFromMemory:MESH(x_file_memptr:Byte Ptr,x_memsize:Int,texture_file_memptr:Byte Ptr,texture_memsize:Int,texture_id:Int,bumpmap_img_memptr:Byte Ptr,bumpmap_img_memsize:Int,bumpmap_id:Int,bConvertToNormalMap:Int,generateFunc:Byte Ptr=Null)
		Local retval:MESH
		If generateFunc<>Null
			Local genFunc:MESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateMesh()
		EndIf
		retval.objptr=_iMeshBumpCreateFromMemory(x_file_memptr,x_memsize,texture_file_memptr,texture_memsize,texture_id,bumpmap_img_memptr,bumpmap_img_memsize,bumpmap_id,bConvertToNormalMap)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod

	Method MeshShadowCasterCreate:MESH(x_filename:String,extrusion:Float,generateFunc:Byte Ptr=Null)
		Local retval:MESH
		If generateFunc<>Null
			Local genFunc:MESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateMesh()
		EndIf
		retval.objptr=_iMeshShadowCasterCreate(x_filename,extrusion)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method MeshShadowCasterCreateFromMemory:MESH(x_filemem_ptr:Byte Ptr,x_memsize:Int,extrusion:Float,generateFunc:Byte Ptr=Null)
		Local retval:MESH
		If generateFunc<>Null
			Local genFunc:MESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateMesh()
		EndIf
		retval.objptr=_iMeshShadowCasterCreateFromMemory(x_filemem_ptr,x_memsize,extrusion)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method MeshBodyShadowCasterCreate:MESH(x_filename:String,bdy:BODY,extrusion:Float,generateFunc:Byte Ptr=Null)
		Local retval:MESH
		If generateFunc<>Null
			Local genFunc:MESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateMesh()
		EndIf
		retval.objptr=_iMeshBodyShadowCasterCreate(x_filename,bdy.objptr,extrusion)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
	
	Method MeshBodyShadowCasterCreateFromMemory:MESH(x_filemem_ptr:Byte Ptr,x_memsize:Int,bdy:BODY,extrusion:Float,generateFunc:Byte Ptr=Null)
		Local retval:MESH
		If generateFunc<>Null
			Local genFunc:MESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateMesh()
		EndIf
		retval.objptr=_iMeshBodyShadowCasterCreateFromMemory(x_filemem_ptr,x_memsize,bdy.objptr,extrusion)
		If retval.objptr<>Null Then Return retval Else Return Null
	EndMethod
		
	Method generateSkinMesh:SKINMESH()
		Return New SKINMESH
	EndMethod

	Method SkinMeshCreate:SKINMESH(x_filename:String,shader_filename:String,generateFunc:Byte Ptr=Null)
		Local retval:SKINMESH
		If generateFunc<>Null
			Local genFunc:SKINMESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateSkinMesh()
		EndIf
		Local shader:Byte Ptr = Null
		If shader_filename Then shader = shader_filename.ToCString()
		retval.objptr=_iSkinMeshCreate(x_filename,shader)
		MemFree(shader)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod
	
	Method SkinMeshCreateFromMemory:SKINMESH(x_filemem_ptr:Byte Ptr,x_memsize:Int,img:Byte Ptr[],imgSize:Int[],shader_filename:String,generateFunc:Byte Ptr=Null)
		Local retval:SKINMESH
		If generateFunc<>Null
			Local genFunc:SKINMESH()=generateFunc
			retval=genFunc()
		Else
			retval=generateSkinMesh()
		EndIf
		Local shader:Byte Ptr = Null
		If shader_filename Then shader = shader_filename.ToCString()
		retval.objptr=_iSkinMeshCreateFromMemory(x_filemem_ptr,x_memsize,Varptr(img[0]),Varptr(imgSize[0]),shader)
		MemFree(shader)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

	Method generateSkinMeshEffect:Byte Ptr(effect_filename:String)
		Return _iSkinMeshEffectCreate(effect_filename)
	EndMethod
		
	Method generateSkinMeshBone:SKINMESHBONE()
		Return New SKINMESHBONE
	EndMethod

	Method generateSubPart:SUBPART()
		Return New SUBPART
	EndMethod

	Method generateEnvMap:ENVMAP()
		Return New ENVMAP
	EndMethod

	Method EnvMapCreate:ENVMAP(dds_filename:String,generateFunc:Byte Ptr=Null)
		Local retval:ENVMAP
		If generateFunc<>Null
			Local genFunc:ENVMAP()=generateFunc
			retval=genFunc()
		Else
			retval=generateEnvMap()
		EndIf
		retval.objptr=_iEnvMapCreate(dds_filename)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod
	
	Method EnvMapCreateFromMemory:ENVMAP(dds_filemem_ptr:Byte Ptr,dds_memsize:Int,generateFunc:Byte Ptr=Null)
		Local retval:ENVMAP
		If generateFunc<>Null
			Local genFunc:ENVMAP()=generateFunc
			retval=genFunc()
		Else
			retval=generateEnvMap()
		EndIf
		retval.objptr=_iEnvMapCreateFromMemory(dds_filemem_ptr,dds_memsize)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

	Method generateSprite:SPRITE()
		Return New SPRITE
	EndMethod

	Method SpriteCreate:SPRITE(x_filename:String,cam:CAMERA,generateFunc:Byte Ptr=Null)
		If cam=Null Then cam=New CAMERA
		Local retval:SPRITE
		If generateFunc<>Null
			Local genFunc:SPRITE()=generateFunc
			retval=genFunc()
		Else
			retval=generateSprite()
		EndIf
		retval.objptr=_iSpriteCreate(x_filename,cam.objptr)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod
	
	Method SpriteCreateFromMemory:SPRITE(x_filemem_ptr:Byte Ptr,x_memsize:Int,texture_filemem_ptr:Byte Ptr,texture_memsize:Int,textureID:Int,cam:CAMERA,generateFunc:Byte Ptr=Null)
		If cam=Null Then cam=New CAMERA
		Local retval:SPRITE
		If generateFunc<>Null
			Local genFunc:SPRITE()=generateFunc
			retval=genFunc()
		Else
			retval=generateSprite()
		EndIf
		retval.objptr=_iSpriteCreateFromMemory(x_filemem_ptr,x_memsize,texture_filemem_ptr,texture_memsize,textureID,cam.objptr)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod
	
	Method generateParticle:PARTICLE()
		Return New PARTICLE
	EndMethod

	Method ParticleCreate:PARTICLE(img_filename:String,maxParticles:Int,generateFunc:Byte Ptr=Null)
		Local retval:PARTICLE
		If generateFunc<>Null
			Local genFunc:PARTICLE()=generateFunc
			retval=genFunc()
		Else
			retval=generateParticle()
		EndIf
		retval.objptr=_iParticleCreate(img_filename,maxParticles)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod
	
	Method ParticleCreateFromMemory:PARTICLE(img_memfile_ptr:Byte Ptr,img_memsize:Int,textureID:Int,maxParticles:Int,generateFunc:Byte Ptr=Null)
		Local retval:PARTICLE
		If generateFunc<>Null
			Local genFunc:PARTICLE()=generateFunc
			retval=genFunc()
		Else
			retval=generateParticle()
		EndIf
		retval.objptr=_iParticleCreateFromMemory(img_memfile_ptr,img_memsize,textureID,maxParticles)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

	Method generateEmitter:EMITTER()
		Return New EMITTER
	EndMethod

	Method generateTrail:TRAIL()
		Return New TRAIL
	EndMethod			
	
	Method TrailCreate:TRAIL(img_filename:String,maxSegments:Int,generateFunc:Byte Ptr=Null)
		Local retval:TRAIL
		If generateFunc<>Null
			Local genFunc:TRAIL()=generateFunc
			retval=genFunc()
		Else
			retval=generateTrail()
		EndIf
		retval.objptr=_iTrailCreate(img_filename,maxSegments)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod
	
	Method TrailCreateFromMemory:TRAIL(img_memfile_ptr:Byte Ptr,img_memsize:Int,textureID:Int,maxSegments:Int,generateFunc:Byte Ptr=Null)
		Local retval:TRAIL
		If generateFunc<>Null
			Local genFunc:TRAIL()=generateFunc
			retval=genFunc()
		Else
			retval=generateTrail()
		EndIf
		retval.objptr=_iTrailCreateFromMemory(img_memfile_ptr,img_memsize,textureID,maxSegments)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

	Method generateTrailEmitter:TRAILEMITTER()
		Return New TRAILEMITTER
	EndMethod																																			

	Method generateTexture:TEXTURE()
		Return New TEXTURE
	EndMethod																																			

	Method TextureCreate:TEXTURE(img_filename:String,generateFunc:Byte Ptr=Null)
		Local retval:TEXTURE
		If generateFunc<>Null
			Local genFunc:TEXTURE()=generateFunc
			retval=genFunc()
		Else
			retval=generateTexture()
		EndIf
		retval.objptr=_iTextureCreate(img_filename)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod
	
	Method TextureCreateFromMemory:TEXTURE(img_memfile_ptr:Byte Ptr,img_memsize:Int,id:Int,generateFunc:Byte Ptr=Null)
		Local retval:TEXTURE
		If generateFunc<>Null
			Local genFunc:TEXTURE()=generateFunc
			retval=genFunc()
		Else
			retval=generateTexture()
		EndIf
		retval.objptr=_iTextureCreateFromMemory(img_memfile_ptr,img_memsize,id)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

	Method generateFont:FONT()
		Return New FONT
	EndMethod
	
	Method FontCreate:FONT(wid_filename:String,cam:CAMERA,generateFunc:Byte Ptr=Null)
		If cam=Null Then cam=New CAMERA
		Local retval:FONT
		If generateFunc<>Null
			Local genFunc:FONT()=generateFunc
			retval=genFunc()
		Else
			retval=generateFont()
		EndIf
		retval.objptr=_iFontCreate(wid_filename,cam.objptr)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

	Method FontCreateFromMemory:FONT(wid_memfile_ptr:Byte Ptr,wid_memsize:Int,x_memfile_ptr:Byte Ptr[],x_memsize:Int[],texture_memfile_ptr:Byte Ptr,texture_memsize:Int,textureID:Int,cam:CAMERA,generateFunc:Byte Ptr=Null)
		If cam=Null Then cam=New CAMERA
		Local retval:FONT
		If generateFunc<>Null
			Local genFunc:FONT()=generateFunc
			retval=genFunc()
		Else
			retval=generateFont()
		EndIf
		retval.objptr=_iFontCreateFromMemory(wid_memfile_ptr,wid_memsize,Varptr(x_memfile_ptr[0]),Varptr(x_memsize[0]),texture_memfile_ptr,texture_memsize,textureID,cam.objptr)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

	Method generateSpring:SPRING()
		Return New SPRING
	EndMethod

	Method SpringCreate:SPRING(bdy:BODY,location:D3DXVECTOR3,targetObject:BODY,springTarget:D3DXVECTOR3,generateFunc:Byte Ptr=Null)
		If targetObject=Null Then targetObject=New BODY
		Local pSpringTarget:Byte Ptr=Null
		If springTarget<>Null Then pSpringTarget=springTarget.param()
		
		Local retval:SPRING
		If generateFunc<>Null
			Local genFunc:SPRING()=generateFunc
			retval=genFunc()
		Else
			retval=generateSpring()
		EndIf
		retval.objptr=_iSpringCreate(bdy.objptr,location.param(),targetObject.objptr,pSpringTarget)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod		
	
	Method generateSound:SOUND()
		Return New SOUND
	EndMethod

	Method SoundCreate:SOUND(pcm_filename:String,b3DSound:Int,generateFunc:Byte Ptr=Null)
		Local retval:SOUND
		If generateFunc<>Null
			Local genFunc:SOUND()=generateFunc
			retval=genFunc()
		Else
			retval=generateSound()
		EndIf
		retval.objptr=_iSoundCreate(pcm_filename,b3DSound)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

	Method SoundOggCreate:SOUND(ogg_filename:String,b3DSound:Int,bufferSize:Int=0,generateFunc:Byte Ptr=Null)
		Local retval:SOUND
		If generateFunc<>Null
			Local genFunc:SOUND()=generateFunc
			retval=genFunc()
		Else
			retval=generateSound()
		EndIf
		retval.objptr=_iSoundOggCreate(ogg_filename,b3DSound,bufferSize)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

	Method SoundOggCreateFromMemory:SOUND(ogg_memfile_ptr:Byte Ptr,ogg_memsize:Int,b3DSound:Int,bufferSize:Int,generateFunc:Byte Ptr=Null)
		Local retval:SOUND
		If generateFunc<>Null
			Local genFunc:SOUND()=generateFunc
			retval=genFunc()
		Else
			retval=generateSound()
		EndIf
		retval.objptr=_iSoundOggCreateFromMemory(ogg_memfile_ptr,ogg_memsize,b3DSound,bufferSize)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

	Method generateBodySync:BODYSYNC()
		Return New BODYSYNC
	EndMethod

	Method BodySyncCreate:BODYSYNC(bdy:BODY,syncFrequency:Float,generateFunc:Byte Ptr=Null)
		Local retval:BODYSYNC
		If generateFunc<>Null
			Local genFunc:BODYSYNC()=generateFunc
			retval=genFunc()
		Else
			retval=generateBodySync()
		EndIf
		retval.objptr=_iNetBodySyncCreate(bdy.objptr,syncFrequency)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod
	
	Method generateFloatSync:FLOATSYNC()
		Return New FLOATSYNC
	EndMethod

	Method FloatSyncCreate:FLOATSYNC(syncVar:Float Ptr,syncFrequency:Float,minSyncFrequency:Float,variationThreshold:Float,generateFunc:Byte Ptr=Null)
		Local retval:FLOATSYNC
		If generateFunc<>Null
			Local genFunc:FLOATSYNC()=generateFunc
			retval=genFunc()
		Else
			retval=generateFloatSync()
		EndIf
		retval.objptr=_iNetFloatSyncCreate(syncVar,syncFrequency,minSyncFrequency,variationThreshold)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod			
	
	Method generateReplayData:REPLAYDATA()
		Return New REPLAYDATA
	EndMethod

	Method ReplayDataCreate:REPLAYDATA(numFloatsToRecord:Int,maxRecordTime:Float,generateFunc:Byte Ptr=Null)
		Local retval:REPLAYDATA
		If generateFunc<>Null
			Local genFunc:REPLAYDATA()=generateFunc
			retval=genFunc()
		Else
			retval=generateReplayData()
		EndIf
		retval.objptr=_iReplayDataCreate(numFloatsToRecord,maxRecordTime)
		If retval.objptr<>Null Then Return retval Else Return Null		
	EndMethod

EndType

' MAIN FUNCTIONS - DLL VERSION ONLY
Function i3ImpactSettings:Int(settings:ISETTINGS)
	If settings=Null Then settings=New SETTINGS
	Return _i3ImpactSettings(settings.param())
EndFunction

Function ggi3ImpactCreateWindow:Byte Ptr(width:Int,height:Int,wName:String,style:Int=WS_POPUP|WS_CAPTION)
	Return _g_3IEngine.CreateWindow(width,height,wName,style)
EndFunction

Function ggi3ImpactCreateFullscreenWindow:Byte Ptr(hWnd:Byte Ptr,wFsName:String,style:Int=WS_POPUP)
	Return _g_3IEngine.CreateFullscreenWindow(hWnd,wFsName,style)
EndFunction

Extern
	' MAIN FUNCTIONS - DLL VERSION ONLY
	Function i3ImpactOpen:Int(windowedHWND:Byte Ptr=Null,fullscreenHWND:Byte Ptr=Null) ' returns true/false
	Function _i3ImpactSettings:Int(ISETTINGS:Byte Ptr)="i3ImpactSettings" ' returns true/false
	Function i3ImpactClose()
	Function i3ImpactIsRendering:Int()
	Function i3ImpactIsActive:Int()
	Function i3ImpactIsReady:Int()
	Function i3ImpactIsExecuting:Int()
	Function i3ImpactIsInitializing:Int()
	Function i3ImpactIsSkippingRunPhase:Int()
	Function i3ImpactIsSkippingRenderPhase:Int()
	Function i3ImpactErrorFlag:Int()
	Function i3ImpactIsRenderingSet(bSet:Int)
	Function i3ImpactIsActiveSet(bSet:Int)
	Function i3ImpactIsReadySet(bSet:Int)
	Function i3ImpactIsExecutingSet(bSet:Int)
	Function i3ImpactIsInitializingSet(bSet:Int)
	Function i3ImpactIsSkippingRunPhaseSet(bSet:Int)
	Function i3ImpactIsSkippingRenderPhaseSet(bSet:Int)
	Function i3ImpactErrorFlagSet(bSet:Int)
	Function i3ImpactMain()
	Function i3ImpactInitBegin()
	Function i3ImpactInitEnd()
	Function i3ImpactRunBegin()
	Function i3ImpactRunEnd()
	Function i3ImpactRender()
	Function i3ImpactExitBegin()
	Function i3ImpactExitEnd()
	Function i3ImpactPreRenderCallbackSet(func(CAMERA:Byte Ptr))
	Function iDisplayWidth:Int()
	Function iDisplayHeight:Int()
		
	Function _i3ImpactCreateWindow:Byte Ptr(width:Int,height:Int,wName$z,style:Int=WS_POPUP|WS_CAPTION)="i3ImpactCreateWindow"
	Function _i3ImpactCreateFullscreenWindow:Byte Ptr(hWnd:Byte Ptr,wFsName$z,style:Int=WS_POPUP)="i3ImpactCreateFullscreenWindow"
	Function ggi3ImpactDestroyWindow(hWnd:Byte Ptr)="i3ImpactDestroyWindow"
	
	Function iRenderingDevice:Byte Ptr()
EndExtern

' special function to bind the callback properly
Extern
	Function bind_PreRenderCallback(func(CAMERA:Byte Ptr))
EndExtern

Function _HANDLE:Byte Ptr(obj:i3ImpactBase)
	If Not obj Then Return Null Else Return obj.objptr
EndFunction