' PARTICLE
' 2006/3/2 GG - added support for v4.2.0 iParticleTextureSet() and iParticleTextureReset()

Type PARTICLE Extends i3ImpactBase	

	Method Destroy()
		_iParticleDestroy(objptr)
		objptr=Null
	EndMethod
	
	Method Hide()
		_iParticleHide(objptr)
	EndMethod
	
	Method Show()
		_iParticleShow(objptr)
	EndMethod
	
	Method RenderModeSet(renderMode:Int)
		_iParticleRenderModeSet(objptr,renderMode)
	EndMethod
	
	Method LifeTimeSet(lifetime:Float)
		_iParticleLifeTimeSet(objptr,lifetime)
	EndMethod
	
	Method AirResistanceSet(airResistance:Float)
		_iParticleAirResistanceSet(objptr,airResistance)
	EndMethod
	
	Method GravitySet(gravity:D3DXVECTOR3)
		_iParticleGravitySet(objptr,gravity.param())
	EndMethod
	
	Method ColorSet(initialRGB:D3DXVECTOR3,finalRGB:D3DXVECTOR3)
		_iParticleColorSet(objptr,initialRGB.param(),finalRGB.param())
	EndMethod
	
	Method Reset()
		_iParticleReset(objptr)
	EndMethod
	
	Method EnvMapCasterAdd()
		_iParticleEnvMapCasterAdd(objptr)
	EndMethod
	
	Method EnvMapCasterRemove()
		_iParticleEnvMapCasterRemove(objptr)
	EndMethod

	Method EmitterCreate:EMITTER(generateFunc:Byte Ptr=Null)
		Local retval:EMITTER
		If generateFunc<>Null
			Local genFunc:EMITTER()=generateFunc
			retval=genFunc()
		Else
			retval=_g_3IEngine.generateEmitter()
		EndIf
		retval.objptr=_iParticleEmitterCreate(objptr)
		If retval.objptr<>Null Then Return retval Else Return Null				
	EndMethod

	Method TextureSet(tex:TEXTURE)	
		_iParticleTextureSet(objptr,tex.objptr)
	EndMethod
	
	Method TextureReset()
		_iParticleTextureReset(objptr)
	EndMethod
	
EndType

Function iParticleCreate:PARTICLE(img_filename:String,maxParticles:Int,generateFunc:Byte Ptr=Null)
	Return _g_3IEngine.ParticleCreate(img_filename,maxParticles,generateFunc) 
EndFunction

Function iParticleCreateFromMemory:PARTICLE(img_memfile_ptr:Byte Ptr,img_memsize:Int,textureID:Int,maxParticles:Int,generateFunc:Byte Ptr=Null)
	Return _g_3IEngine.ParticleCreateFromMemory(img_memfile_ptr,img_memsize,textureID,maxParticles,generateFunc) 
EndFunction

Function iParticleDestroy(part:PARTICLE)
	part.Destroy()
EndFunction

Function iParticleHide(part:PARTICLE)
	part.Hide()
EndFunction

Function iParticleShow(part:PARTICLE)
	part.Show()
EndFunction

Function iParticleTextureSet(part:PARTICLE,tex:TEXTURE)
	part.TextureSet(tex)
EndFunction

Function iParticleTextureReset(part:PARTICLE)
	part.TextureReset()
EndFunction

Function iParticleRenderModeSet(part:PARTICLE,renderMode:Int)
	part.RenderModeSet(renderMode)
EndFunction

Function iParticleLifeTimeSet(part:PARTICLE,lifetime:Float)
	part.LifeTimeSet(lifetime)
EndFunction

Function iParticleAirResistanceSet(part:PARTICLE,airResistance:Float)
	part.AirResistanceSet(airResistance)
EndFunction

Function iParticleGravitySet(part:PARTICLE,gravity:D3DXVECTOR3)
	part.GravitySet(gravity)
EndFunction

Function iParticleColorSet(part:PARTICLE,initialRGB:D3DXVECTOR3,finalRGB:D3DXVECTOR3)
	part.ColorSet(initialRGB,finalRGB)
EndFunction

Function iParticleReset(part:PARTICLE)
	part.Reset()
EndFunction

Function iParticleEnvMapCasterAdd(part:PARTICLE)
	part.EnvMapCasterAdd()
EndFunction

Function iParticleEnvMapCasterRemove(part:PARTICLE)
	part.EnvMapCasterRemove()
EndFunction

Function iParticleEmitterCreate:EMITTER(part:PARTICLE,generateFunc:Byte Ptr=Null)
	Return part.EmitterCreate(generateFunc)
EndFunction

Type EMITTER Extends i3ImpactBase	

	Method Destroy()
		_iParticleEmitterDestroy(objptr)
		objptr=Null
	EndMethod
	
	Method Emit(particlesPerSecond:Float,initialAlphaIntensity:Float)
		_iParticleEmitterEmit(objptr,particlesPerSecond,initialAlphaIntensity)
	EndMethod
	
	Method LocationSet(location:D3DXVECTOR3)
		_iParticleEmitterLocationSet(objptr,location.param())
	EndMethod
	
	Method DirectionSet(minDirection:D3DXVECTOR3,maxDirection:D3DXVECTOR3)
		_iParticleEmitterDirectionSet(objptr,minDirection.param(),maxDirection.param())
	EndMethod
	
	Method SpeedSet(minSpeed:Float,maxSpeed:Float)
		_iParticleEmitterSpeedSet(objptr,minSpeed,maxSpeed)
	EndMethod
	
	Method ScaleSet(initialSize:D3DXVECTOR2,finalSize:D3DXVECTOR2)
		_iParticleEmitterScaleSet(objptr,initialSize.param(),finalSize.param())
	EndMethod
	
EndType

Function iParticleEmitterDestroy(emit:EMITTER)
	emit.Destroy()
EndFunction

Function iParticleEmitterEmit(emit:EMITTER,particlesPerSecond:Float,initialAlphaIntensity:Float)
	emit.Emit(particlesPerSecond,initialAlphaIntensity)
EndFunction

Function iParticleEmitterLocationSet(emit:EMITTER,location:D3DXVECTOR3)
	emit.LocationSet(location)
EndFunction

Function iParticleEmitterDirectionSet(emit:EMITTER,minDirection:D3DXVECTOR3,maxDirection:D3DXVECTOR3)
	emit.DirectionSet(minDirection,maxDirection)
EndFunction

Function iParticleEmitterSpeedSet(emit:EMITTER,minSpeed:Float,maxSpeed:Float)
	emit.SpeedSet(minSpeed,maxSpeed)
EndFunction

Function iParticleEmitterScaleSet(emit:EMITTER,initialSize:D3DXVECTOR2,finalSize:D3DXVECTOR2)
	emit.ScaleSet(initialSize,finalSize)
EndFunction

Extern
	' PARTICLE
	Function _iParticleCreate:Byte Ptr(img_filename$z,maxParticles:Int)="iParticleCreate" ' returns PARTICLE*
	Function _iParticleCreateFromMemory:Byte Ptr(img_memfile_ptr:Byte Ptr,img_memsize:Int,textureID:Int,maxParticles:Int)="iParticleCreateFromMemory" ' returns PARTICLE*

	Function _iParticleDestroy(PARTICLE:Byte Ptr)="iParticleDestroy"
	Function _iParticleHide(PARTICLE:Byte Ptr)="iParticleHide"
	Function _iParticleShow(PARTICLE:Byte Ptr)="iParticleShow"
	Function _iParticleRenderModeSet(PARTICLE:Byte Ptr,renderMode:Int)="iParticleRenderModeSet"
	Function _iParticleLifeTimeSet(PARTICLE:Byte Ptr,lifetime:Float)="iParticleLifeTimeSet"
	Function _iParticleAirResistanceSet(PARTICLE:Byte Ptr,airResistance:Float)="iParticleAirResistanceSet"
	Function _iParticleGravitySet(PARTICLE:Byte Ptr,D3DXVECTOR3_gravity:Byte Ptr)="iParticleGravitySet"
	Function _iParticleColorSet(PARTICLE:Byte Ptr,D3DXVECTOR3_initialRGB:Byte Ptr,D3DXVECTOR3_finalRGB:Byte Ptr)="iParticleColorSet"
	Function _iParticleReset(PARTICLE:Byte Ptr)="iParticleReset"
	Function _iParticleEmitterCreate:Byte Ptr(PARTICLE:Byte Ptr)="iParticleEmitterCreate" ' returns EMITTER* 
	Function _iParticleEmitterDestroy(EMITTER:Byte Ptr)="iParticleEmitterDestroy"
	Function _iParticleEmitterEmit(EMITTER:Byte Ptr,particlesPerSecond:Float,initialAlphaIntensity:Float)="iParticleEmitterEmit"
	Function _iParticleEmitterLocationSet(EMITTER:Byte Ptr,D3DXVECTOR3_location:Byte Ptr)="iParticleEmitterLocationSet"
	Function _iParticleEmitterDirectionSet(EMITTER:Byte Ptr,D3DXVECTOR3_minDirection:Byte Ptr,D3DXVECTOR3_maxDirection:Byte Ptr)="iParticleEmitterDirectionSet"
	Function _iParticleEmitterSpeedSet(EMITTER:Byte Ptr,minSpeed:Float,maxSpeed:Float)="iParticleEmitterSpeedSet"
	Function _iParticleEmitterScaleSet(EMITTER:Byte Ptr,D3DXVECTOR2_initialSize:Byte Ptr,D3DXVECTOR2_finalSize:Byte Ptr)="iParticleEmitterScaleSet"
	Function _iParticleEnvMapCasterAdd(PARTICLE:Byte Ptr)="iParticleEnvMapCasterAdd"
	Function _iParticleEnvMapCasterRemove(PARTICLE:Byte Ptr)="iParticleEnvMapCasterRemove"
	Function _iParticleTextureSet(PARTICLE:Byte Ptr,TEXTURE:Byte Ptr)="iParticleTextureSet"
	Function _iParticleTextureReset(PARTICLE:Byte Ptr)="iParticleTextureReset"
	
EndExtern
